@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2004-2006 Imendio AB
+ * Copyright (C) 2004-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,31 +18,30 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
+
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+
 #include <libxml/parser.h>
 #include <libxml/tree.h>
 
-#include "libgossip-marshal.h"
-#include "gossip-account-manager.h"
 #include "gossip-debug.h"
-#include "gossip-protocol.h"
+#include "gossip-jabber.h"
+#include "gossip-account-manager.h"
+#include "gossip-private.h"
 #include "gossip-utils.h"
 
+#include "libgossip-marshal.h"
+
 #define DEBUG_DOMAIN "AccountManager"
 
-#define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_ACCOUNT_MANAGER, GossipAccountManagerPriv))
-
-/* For splitting an id of user@server/resource to just user@server with resource
- * in it's own xml tags (for Gossip release 0.10.2).
- */
-#define RESOURCE_HACK
-
 #define ACCOUNTS_XML_FILENAME "accounts.xml"
 #define ACCOUNTS_DTD_FILENAME "gossip-account.dtd"
 
+#define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_ACCOUNT_MANAGER, GossipAccountManagerPriv))
+
 typedef struct _GossipAccountManagerPriv GossipAccountManagerPriv;
 
 struct _GossipAccountManagerPriv {
@@ -66,17 +65,9 @@ static gboolean account_manager_get_all                (GossipAccountManager *ma
 static gboolean account_manager_file_parse             (GossipAccountManager *manager,
 							const gchar          *filename);
 static gboolean account_manager_file_save              (GossipAccountManager *manager);
-static void     account_manager_add_parameters_foreach (GossipAccount        *account,
-							gchar                *param_name,
-							GossipAccountParam   *param,
-							xmlNodePtr            node);
 
 static guint signals[LAST_SIGNAL] = {0};
 
-#ifdef RESOURCE_HACK
-static gboolean need_saving = FALSE;
-#endif
-
 G_DEFINE_TYPE (GossipAccountManager, gossip_account_manager, G_TYPE_OBJECT);
 
 static void
@@ -167,6 +158,8 @@ gossip_account_manager_add (GossipAccountManager *manager,
 			    GossipAccount        *account)
 {
 	GossipAccountManagerPriv *priv;
+	GList                    *l;
+	const gchar              *name;
 
 	g_return_val_if_fail (GOSSIP_IS_ACCOUNT_MANAGER (manager), FALSE);
 	g_return_val_if_fail (GOSSIP_IS_ACCOUNT (account), FALSE);
@@ -174,25 +167,21 @@ gossip_account_manager_add (GossipAccountManager *manager,
 	priv = GET_PRIV (manager);
 
 	/* Don't add more than once */
-	if (!gossip_account_manager_find (manager, gossip_account_get_name (account))) {
-		const gchar       *name;
-		GossipAccountType  type;
-
-		type = gossip_account_get_type (account);
-		name = gossip_account_get_name (account);
-
-		gossip_debug (DEBUG_DOMAIN, "Adding %s account with name:'%s'",
-			      gossip_account_type_to_string (type),
-			      name);
-
-		priv->accounts = g_list_append (priv->accounts, g_object_ref (account));
-
-		g_signal_emit (manager, signals[ACCOUNT_ADDED], 0, account);
-
-		return TRUE;
+	for (l = priv->accounts; l; l = l->next) {
+		if (gossip_account_equal (l->data, account)) {
+			return FALSE;
+		}
 	}
 
-	return FALSE;
+	priv->accounts = g_list_append (priv->accounts, g_object_ref (account));
+	gossip_account_manager_set_unique_name (manager, account);
+
+	name = gossip_account_get_name (account);
+	gossip_debug (DEBUG_DOMAIN, "Adding account with name:'%s'", name);
+
+	g_signal_emit (manager, signals[ACCOUNT_ADDED], 0, account);
+
+	return TRUE;
 }
 
 void
@@ -228,7 +217,7 @@ gossip_account_manager_get_accounts (GossipAccountManager *manager)
 	priv = GET_PRIV (manager);
 
 	accounts = g_list_copy (priv->accounts);
-	g_list_foreach (accounts, (GFunc)g_object_ref, NULL);
+	g_list_foreach (accounts, (GFunc) g_object_ref, NULL);
 
 	return accounts;
 }
@@ -281,22 +270,19 @@ gossip_account_manager_find_by_id (GossipAccountManager *manager,
 	GList                    *l;
 
 	g_return_val_if_fail (GOSSIP_IS_ACCOUNT_MANAGER (manager), NULL);
-	g_return_val_if_fail (id != NULL, NULL);
+	g_return_val_if_fail (!G_STR_EMPTY (id), NULL);
 
 	priv = GET_PRIV (manager);
 
 	for (l = priv->accounts; l; l = l->next) {
 		GossipAccount *account;
-		const gchar   *account_id;
+		const gchar   *this_id = NULL;
 
 		account = l->data;
-		account_id = gossip_account_get_id (account);
 
-		if (!account_id) {
-			continue;
-		}
+		this_id = gossip_account_get_id (account);
 
-		if (strcmp (account_id, id) == 0) {
+		if (this_id && strcmp (id, this_id) == 0) {
 			return account;
 		}
 	}
@@ -378,17 +364,12 @@ gossip_account_manager_get_default (GossipAccountManager *manager)
 		/* if one or more entries, use that */
 		if (gossip_account_manager_get_count (manager) >= 1) {
 			GossipAccount *account;
-			GList         *l;
 
-			l = gossip_account_manager_get_accounts (manager);
-			account = l->data;
+			account = priv->accounts->data;
 
 			name = gossip_account_get_name (account);
 			gossip_account_manager_set_default (manager, account);
 
-			g_list_foreach (l, (GFunc) g_object_unref, NULL);
-			g_list_free (l);
-
 			/* We return the default account ref here. */
 			return account;
 		}
@@ -435,7 +416,7 @@ account_manager_get_all (GossipAccountManager *manager)
 		file_with_path = g_strdup (priv->accounts_file_name);
 	}
 
-	/* read file in */
+	/* Read file in */
 	if (g_file_test (file_with_path, G_FILE_TEST_EXISTS) &&
 	    !account_manager_file_parse (manager, file_with_path)) {
 		g_free (file_with_path);
@@ -448,25 +429,13 @@ account_manager_get_all (GossipAccountManager *manager)
 }
 
 static void
-account_manager_parse_account (GossipAccountManager *manager,
-			       xmlNodePtr            node)
+account_manager_parse_account (GossipAccountManager *manager, xmlNodePtr node)
 {
-	GossipAccount  *account;
-	GossipProtocol *protocol;
-	xmlNodePtr      child;
-	gchar          *str;
+	GossipAccount *account;
+	xmlNodePtr     child;
+	gchar         *str;
 
-	str = xmlGetProp (node, "type");
-	if (str) {
-		protocol = gossip_protocol_new_from_account_type (gossip_account_string_to_type (str));
-	} else {
-		protocol = gossip_protocol_new_from_account_type (GOSSIP_ACCOUNT_TYPE_JABBER);
-	}
-
-	account = gossip_protocol_new_account (protocol);
-
-	g_object_unref (protocol);
-	xmlFree (str);
+	account = gossip_jabber_new_account ();
 
 	child = node->children;
 	while (child) {
@@ -481,113 +450,33 @@ account_manager_parse_account (GossipAccountManager *manager,
 
 		if (strcmp (tag, "name") == 0) {
 			gossip_account_set_name (account, str);
-		}
-		else if (strcmp (tag, "id") == 0) {
+		} else if (strcmp (tag, "id") == 0) {
 			gossip_account_set_id (account, str);
-		}
-		else if (strcmp (tag, "auto_connect") == 0) {
+		} else if (strcmp (tag, "resource") == 0) {
+			gossip_account_set_resource (account, str);
+		} else if (strcmp (tag, "password") == 0) {
+			gossip_account_set_password (account, str);
+		} else if (strcmp (tag, "server") == 0) {
+			gossip_account_set_server (account, str);
+		} else if (strcmp (tag, "port") == 0) {
+			guint port;
+
+			port = atoi (str);
+			if (port != 0) {
+				gossip_account_set_port (account, port);
+			}
+		} else if (strcmp (tag, "auto_connect") == 0) {
 			gossip_account_set_auto_connect (account, strcmp (str, "yes") == 0);
-		}
-		else if (strcmp (tag, "use_proxy") == 0) {
+		} else if (strcmp (tag, "use_ssl") == 0) {
+			gossip_account_set_use_ssl (account, strcmp (str, "yes") == 0);
+		} else if (strcmp (tag, "use_proxy") == 0) {
 			gossip_account_set_use_proxy (account, strcmp (str, "yes") == 0);
 		}
-		else if (strcmp (tag, "type") == 0) {
-			g_object_set (account,
-				      "type",
-				      gossip_account_string_to_type (str),
-				      NULL);
-		}
-		else if (strcmp (tag, "parameter") == 0) {
-			gchar                   *param_name;
-			gchar                   *type_str;
-			GType                    type;
-			GValue                  *g_value;
-
-			param_name = xmlGetProp (child, "name");
-			type_str = xmlGetProp (child, "type");
-
-			type = gossip_dbus_type_to_g_type (type_str);
-			g_value = gossip_string_to_g_value (str, type);
-
-			/* Compatibility: The ID was defined here
-			 * in old accounts.xml files. */
-			if (strcmp (param_name, "id") != 0) {
-				gossip_account_param_set_g_value (account,
-								  param_name,
-								  g_value);			
-			} else {
-				/* Save in the new format */
-				need_saving = TRUE;
-				gossip_account_set_id (account, str);
-			}
-
-			g_value_unset (g_value);
-			g_free (g_value);
-			xmlFree (param_name);
-			xmlFree (type_str);
-		}
-		/* Those are deprecated and kept for compatibility only */
-		else if (strcmp (tag, "resource") == 0) {
-			gossip_account_param_set (account, "resource", str, NULL);
-		}
-		else if (strcmp (tag, "password") == 0) {
-			gossip_account_param_set (account, "password", str, NULL);
-		}
-		else if (strcmp (tag, "server") == 0) {
-			gossip_account_param_set (account, "server", str, NULL);
-		}
-		else if (strcmp (tag, "port") == 0) {
-			guint tmp_port;
-
-			tmp_port = atoi (str);
-			if (tmp_port != 0) {
-				gossip_account_param_set (account, "port",
-							  tmp_port, NULL);
-			}
-		}
-		else if (strcmp (tag, "use_ssl") == 0) {
-			gossip_account_param_set (account, "use_ssl",
-						  strcmp (str, "yes") == 0,
-						  NULL);
-		}
 
 		xmlFree (str);
 
 		child = child->next;
 	}
-#ifdef RESOURCE_HACK
-	const gchar *resource_found = NULL;
-	const gchar *id;
-
-	id = gossip_account_get_id (account);
-	str = g_strdup (id);
-	if (str) {
-		/* FIXME: This hack is so we don't get bug
-		 * reports and basically gets the resource
-		 * from the id.
-		 */
-		const gchar *ch;
-
-		ch = strchr (str, '/');
-		if (ch) {
-			resource_found = (ch + 1);
-			str[ch - str] = '\0';
-
-			g_printerr ("Converting ID... (id:'%s', resource:'%s')\n",
-				    str, resource_found);
-		}
-	}
-
-	if (resource_found) {
-		gossip_account_param_set (account,
-					  "id", str,
-					  "resource", resource_found,
-					  NULL);
-		need_saving = TRUE;
-	}
-
-	g_free (str);
-#endif /* RESOURCE_HACK */
 
 	gossip_account_manager_add (manager, account);
 
@@ -660,13 +549,6 @@ account_manager_file_parse (GossipAccountManager *manager,
 	xmlFreeDoc(doc);
 	xmlFreeParserCtxt (ctxt);
 
-#ifdef RESOURCE_HACK
-	if (need_saving) {
-		g_printerr ("Saving accounts... \n");
-		account_manager_file_save (manager);
-	}
-#endif
-
 	return TRUE;
 }
 
@@ -675,7 +557,9 @@ account_manager_file_save (GossipAccountManager *manager)
 {
 	GossipAccountManagerPriv *priv;
 	xmlDocPtr                 doc;
+	xmlDocPtr                 old_doc;
 	xmlNodePtr                root;
+	xmlParserCtxtPtr          ctxt;
 	GList                    *accounts;
 	GList                    *l;
 	gchar                    *xml_dir;
@@ -702,31 +586,51 @@ account_manager_file_save (GossipAccountManager *manager)
 		priv->default_name = g_strdup ("Default");
 	}
 
-	xmlNewChild (root, NULL,
-		    "default",
-		    priv->default_name);
+	xmlNewChild (root, NULL, "default", priv->default_name);
+
+	/* Copy not used accounts in the new document */
+	ctxt = xmlNewParserCtxt ();
+	old_doc = xmlCtxtReadFile (ctxt, xml_file, NULL, 0);
+
+	if (old_doc) {
+		xmlNodePtr node;
+
+		node = xmlDocGetRootElement (old_doc);
+		for (node = node->children; node; node = node->next) {
+			if (strcmp ((gchar *) node->name, "default") == 0) {
+				continue;
+			}
+		}
+
+		xmlFreeDoc(old_doc);
+	}
+
+	xmlFreeParserCtxt (ctxt);
 
 	accounts = gossip_account_manager_get_accounts (manager);
-
 	for (l = accounts; l; l = l->next) {
-		GossipAccount           *account;
-		const gchar             *type;
-		xmlNodePtr               node;
-
+		GossipAccount *account;
+		gchar         *port;
+		xmlNodePtr     node;
+	
 		account = l->data;
 
-		type = gossip_account_type_to_string (gossip_account_get_type (account));
+		port = g_strdup_printf ("%d", gossip_account_get_port (account));
 
 		node = xmlNewChild (root, NULL, "account", NULL);
-		xmlNewProp (node, "type", type);
 		xmlNewTextChild (node, NULL, "name", gossip_account_get_name (account));
 		xmlNewTextChild (node, NULL, "id", gossip_account_get_id (account));
-		xmlNewTextChild (node, NULL, "auto_connect", gossip_account_get_auto_connect (account) ? "yes" : "no");
-		xmlNewTextChild (node, NULL, "use_proxy", gossip_account_get_use_proxy (account) ? "yes" : "no");
 
-		gossip_account_param_foreach (account,
-					      (GossipAccountParamFunc) account_manager_add_parameters_foreach,
-					      node);
+		xmlNewTextChild (node, NULL, "resource", gossip_account_get_resource (account));
+		xmlNewTextChild (node, NULL, "password", gossip_account_get_password (account));
+		xmlNewTextChild (node, NULL, "server", gossip_account_get_server (account));
+		xmlNewChild (node, NULL, "port", port);
+
+		xmlNewChild (node, NULL, "auto_connect", gossip_account_get_auto_connect (account) ? "yes" : "no");
+		xmlNewChild (node, NULL, "use_ssl", gossip_account_get_use_ssl (account) ? "yes" : "no");
+		xmlNewChild (node, NULL, "use_proxy", gossip_account_get_use_proxy (account) ? "yes" : "no");
+
+		g_free (port);
 	}
 
 	gossip_debug (DEBUG_DOMAIN, "Saving file:'%s'", xml_file);
@@ -737,7 +641,13 @@ account_manager_file_save (GossipAccountManager *manager)
 	 */
 	old_mask = umask (077);
 	chmod (xml_file, 0600);
+
+	/* Make sure the XML is indented properly */
+	xmlIndentTreeOutput = 1;
+
 	xmlSaveFormatFileEnc (xml_file, doc, "utf-8", 1);
+
+	/* Reset the umask */
 	umask (old_mask);
 
 	xmlFreeDoc (doc);
@@ -752,21 +662,37 @@ account_manager_file_save (GossipAccountManager *manager)
 	return TRUE;
 }
 
-static void
-account_manager_add_parameters_foreach (GossipAccount      *account,
-					gchar              *param_name,
-					GossipAccountParam *param,
-					xmlNodePtr          node)
+gboolean
+gossip_account_manager_set_unique_name (GossipAccountManager *manager,
+					GossipAccount        *account)
 {
-	gchar       *value_str;
-	const gchar *type_str;
+	GossipAccountManagerPriv *priv;
+	GList                    *l;
+	gchar                    *new_name;
 
-	value_str = gossip_g_value_to_string (&param->g_value);
-	type_str = gossip_g_type_to_dbus_type (G_VALUE_TYPE (&param->g_value));
+	g_return_val_if_fail (GOSSIP_IS_ACCOUNT_MANAGER (manager), FALSE);
 
-	node = xmlNewTextChild (node, NULL, "parameter", value_str);
-	xmlNewProp (node, "name", param_name);
-	xmlNewProp (node, "type", type_str);
+	priv = GET_PRIV (manager);
 
-	g_free (value_str);
+	new_name = g_strdup (gossip_account_get_name (account));
+	l = priv->accounts;
+	while (l) {
+		if (l->data != account &&
+		    strcmp (new_name, gossip_account_get_name (l->data)) == 0) {
+			gchar *str;
+
+			str = g_strdup_printf ("%s_", new_name);
+			g_free (new_name);
+			new_name = str;
+
+			l = priv->accounts;
+			continue;
+		}
+		l = l->next;
+	}
+	gossip_account_set_name (account, new_name);
+	g_free (new_name);
+
+	return TRUE;
 }
+
