@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2004 Imendio AB
+ * Copyright (C) 2004-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,21 +16,25 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ * 
  */
 
-#include <config.h>
+#include "config.h"
+
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+
 #include <libxml/parser.h>
 #include <libxml/tree.h>
 
-#include "libgossip-marshal.h"
-
-#include "gossip-utils.h"
 #include "gossip-debug.h"
 #include "gossip-chatroom-manager.h"
-#include "gossip-protocol.h"
+#include "gossip-account-manager.h"
+#include "gossip-private.h"
+#include "gossip-utils.h"
+
+#include "libgossip-marshal.h"
 
 #define DEBUG_DOMAIN "ChatroomManager"
 
@@ -43,10 +47,8 @@ typedef struct _GossipChatroomManagerPriv GossipChatroomManagerPriv;
 
 struct _GossipChatroomManagerPriv {
 	GossipAccountManager *account_manager;
-	GossipSession        *session;
 
 	GList                *chatrooms;
-	GList                *chatrooms_to_start; /* for auto connect */
 
 	gchar                *chatrooms_file_name;
 
@@ -64,20 +66,11 @@ static gboolean chatroom_manager_get_all               (GossipChatroomManager
 static gboolean chatroom_manager_file_parse            (GossipChatroomManager    *manager,
 							const gchar              *filename);
 static gboolean chatroom_manager_file_save             (GossipChatroomManager    *manager);
-static void     chatroom_manager_join_cb               (GossipChatroomProvider   *provider,
-							GossipChatroomJoinResult  result,
-							GossipChatroomId          id,
-							GossipChatroomManager    *manager);
-static void     chatroom_manager_protocol_connected_cb (GossipSession            *session,
-							GossipAccount            *account,
-							GossipProtocol           *protocol,
-							GossipChatroomManager    *manager);
 
 enum {
 	CHATROOM_ADDED,
 	CHATROOM_REMOVED,
 	CHATROOM_ENABLED,
-	CHATROOM_AUTO_CONNECT_UPDATE,
 	CHATROOM_FAVOURITE_UPDATE,
 	NEW_DEFAULT,
 	LAST_SIGNAL
@@ -121,18 +114,6 @@ gossip_chatroom_manager_class_init (GossipChatroomManagerClass *klass)
 			      libgossip_marshal_VOID__OBJECT,
 			      G_TYPE_NONE,
 			      1, GOSSIP_TYPE_CHATROOM);
-	signals[CHATROOM_AUTO_CONNECT_UPDATE] =
-		g_signal_new ("chatroom-auto-connect-update",
-			      G_TYPE_FROM_CLASS (klass),
-			      G_SIGNAL_RUN_LAST,
-			      0,
-			      NULL, NULL,
-			      libgossip_marshal_VOID__OBJECT_OBJECT_INT,
-			      G_TYPE_NONE,
-			      3,
-			      GOSSIP_TYPE_CHATROOM_PROVIDER,
-			      GOSSIP_TYPE_CHATROOM,
-			      G_TYPE_INT);
 	signals[CHATROOM_FAVOURITE_UPDATE] =
 		g_signal_new ("chatroom-favourite-update",
 			      G_TYPE_FROM_CLASS (klass),
@@ -171,21 +152,10 @@ chatroom_manager_finalize (GObject *object)
 	g_list_foreach (priv->chatrooms, (GFunc)g_object_unref, NULL);
 	g_list_free (priv->chatrooms);
 
-	g_list_foreach (priv->chatrooms_to_start, (GFunc)g_object_unref, NULL);
-	g_list_free (priv->chatrooms_to_start);
-
 	if (priv->account_manager) {
 		g_object_unref (priv->account_manager);
 	}
 
-	if (priv->session) {
-		g_signal_handlers_disconnect_by_func (priv->session,
-						      chatroom_manager_protocol_connected_cb,
-						      object);
-
-		g_object_unref (priv->session);
-	}
-
 	g_free (priv->chatrooms_file_name);
 	g_free (priv->default_name);
 
@@ -194,7 +164,6 @@ chatroom_manager_finalize (GObject *object)
 
 GossipChatroomManager *
 gossip_chatroom_manager_new (GossipAccountManager *account_manager,
-			     GossipSession        *session,
 			     const gchar          *filename)
 {
 
@@ -202,7 +171,6 @@ gossip_chatroom_manager_new (GossipAccountManager *account_manager,
 	GossipChatroomManagerPriv *priv;
 
 	g_return_val_if_fail (GOSSIP_IS_ACCOUNT_MANAGER (account_manager), NULL);
-	g_return_val_if_fail (GOSSIP_IS_SESSION (session), NULL);
 
 	manager = g_object_new (GOSSIP_TYPE_CHATROOM_MANAGER, NULL);
 
@@ -212,23 +180,13 @@ gossip_chatroom_manager_new (GossipAccountManager *account_manager,
 		priv->account_manager = g_object_ref (account_manager);
 	}
 
-	if (session) {
-		priv->session = g_object_ref (session);
-	}
-
 	if (filename) {
 		priv->chatrooms_file_name = g_strdup (filename);
 	}
 
-	/* load file */
+	/* Load file */
 	chatroom_manager_get_all (manager);
 
-	/* connect protocol connected/disconnected signals */
-	g_signal_connect (priv->session,
-			  "protocol-connected",
-			  G_CALLBACK (chatroom_manager_protocol_connected_cb),
-			  manager);
-
 	return manager;
 }
 
@@ -267,11 +225,6 @@ gossip_chatroom_manager_add (GossipChatroomManager *manager,
 		priv->chatrooms = g_list_append (priv->chatrooms,
 						 g_object_ref (chatroom));
 
-		if (gossip_chatroom_get_auto_connect (chatroom)) {
-			priv->chatrooms_to_start = g_list_append (priv->chatrooms_to_start,
-								  g_object_ref (chatroom));
-		}
-
 		g_signal_emit (manager, signals[CHATROOM_ADDED], 0, chatroom);
 
 		return TRUE;
@@ -285,6 +238,7 @@ gossip_chatroom_manager_remove (GossipChatroomManager *manager,
 				GossipChatroom        *chatroom)
 {
 	GossipChatroomManagerPriv *priv;
+	GList                     *link;
 
 	g_return_if_fail (GOSSIP_IS_CHATROOM_MANAGER (manager));
 	g_return_if_fail (GOSSIP_IS_CHATROOM (chatroom));
@@ -303,12 +257,12 @@ gossip_chatroom_manager_remove (GossipChatroomManager *manager,
 					      chatroom_manager_chatroom_favourite_cb,
 					      manager);
 
-	priv->chatrooms = g_list_remove (priv->chatrooms, chatroom);
-	priv->chatrooms_to_start = g_list_remove (priv->chatrooms_to_start, chatroom);
-
-	g_signal_emit (manager, signals[CHATROOM_REMOVED], 0, chatroom);
-
-	g_object_unref (chatroom);
+	link = g_list_find (priv->chatrooms, chatroom);
+	if (link) {
+		priv->chatrooms = g_list_delete_link (priv->chatrooms, link);
+		g_signal_emit (manager, signals[CHATROOM_REMOVED], 0, chatroom);
+		g_object_unref (chatroom);
+	}
 }
 
 static void
@@ -537,42 +491,6 @@ gossip_chatroom_manager_store (GossipChatroomManager *manager)
 	return chatroom_manager_file_save (manager);
 }
 
-void
-gossip_chatroom_manager_join_favourites (GossipChatroomManager *manager)
-{
-	GossipChatroomManagerPriv *priv;
-	GList                     *l;
-
-	g_return_if_fail (GOSSIP_IS_CHATROOM_MANAGER (manager));
-
-	priv = GET_PRIV (manager);
-
-	for (l = priv->chatrooms; l; l = l->next) {
-		GossipChatroomProvider *provider;
-		GossipChatroom         *chatroom;
-		GossipAccount          *account;
-
-		chatroom = l->data;
-
-		if (!gossip_chatroom_get_is_favourite (chatroom)) {
-			continue;
-		}
-
-		account = gossip_chatroom_get_account (chatroom);
-
-		if (!gossip_session_is_connected (priv->session, account)) {
-			continue;
-		}
-
-		provider = gossip_session_get_chatroom_provider (priv->session, account);
-
-		gossip_chatroom_provider_join (provider,
-					       chatroom,
-					       (GossipChatroomJoinCb) chatroom_manager_join_cb,
-					       manager);
-	}
-}
-
 /*
  * API to save/load and parse the chatrooms file.
  */
@@ -642,6 +560,11 @@ chatroom_manager_parse_chatroom (GossipChatroomManager *manager,
 	while (child) {
 		gchar *tag;
 
+		if (xmlNodeIsText (child)) {
+			child = child->next;
+			continue;
+		}
+
 		tag = (gchar *) child->name;
 		str = (gchar *) xmlNodeGetContent (child);
 
@@ -860,7 +783,7 @@ chatroom_manager_file_save (GossipChatroomManager *manager)
 
 		xmlNewTextChild (node, NULL, "password", gossip_chatroom_get_password (chatroom));
 		xmlNewChild (node, NULL, "auto_connect", gossip_chatroom_get_auto_connect (chatroom) ? "yes" : "no");
-		xmlNewChild (node, NULL, "favourite", gossip_chatroom_get_is_favourite (chatroom) ? "yes" : "no");
+		xmlNewChild (node, NULL, "favourite", gossip_chatroom_get_favourite (chatroom) ? "yes" : "no");
 
 		account = gossip_chatroom_get_account (chatroom);
 		if (account) {
@@ -870,6 +793,9 @@ chatroom_manager_file_save (GossipChatroomManager *manager)
 		g_free (type);
 	}
 
+	/* Make sure the XML is indented properly */
+	xmlIndentTreeOutput = 1;
+
 	gossip_debug (DEBUG_DOMAIN, "Saving file:'%s'", file);
 	xmlSaveFormatFileEnc (file, doc, "utf-8", 1);
 	xmlFreeDoc (doc);
@@ -883,59 +809,3 @@ chatroom_manager_file_save (GossipChatroomManager *manager)
 
 	return TRUE;
 }
-
-static void
-chatroom_manager_join_cb (GossipChatroomProvider   *provider,
-			  GossipChatroomJoinResult  result,
-			  GossipChatroomId          id,
-			  GossipChatroomManager    *manager)
-{
-	GossipChatroom *chatroom;
-
-	chatroom = gossip_chatroom_provider_find (provider, id);
-
-	g_signal_emit (manager, signals[CHATROOM_AUTO_CONNECT_UPDATE], 0,
-		       provider, chatroom, result);
-}
-
-static void
-chatroom_manager_protocol_connected_cb (GossipSession         *session,
-					GossipAccount         *account,
-					GossipProtocol        *protocol,
-					GossipChatroomManager *manager)
-{
-	GossipChatroomManagerPriv *priv;
-	GList                     *chatrooms, *l;
-
-	priv = GET_PRIV (manager);
-
-	gossip_debug (DEBUG_DOMAIN,
-		      "Account:'%s' connected, checking chatroom auto connects.",
-		      gossip_account_get_name (account));
-
-	chatrooms = gossip_chatroom_manager_get_chatrooms (manager, account);
-
-	for (l = chatrooms; l; l = l->next) {
-		GossipChatroom *chatroom;
-
-		chatroom = l->data;
-
-		if (g_list_find (priv->chatrooms_to_start, chatroom)) {
-			GossipChatroomProvider *provider;
-
-			/* found in list, it needs to be connected to */
-			provider = gossip_session_get_chatroom_provider (session, account);
-
-			gossip_chatroom_provider_join (provider,
-						       chatroom,
-						       (GossipChatroomJoinCb) chatroom_manager_join_cb,
-						       manager);
-
-			/* take out of list */
-			priv->chatrooms_to_start = g_list_remove (priv->chatrooms_to_start,
-								  chatroom);
-		}
-	}
-
-	g_list_free (chatrooms);
-}
