@@ -16,8 +16,6 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
- *
- * Authors: Martyn Russell <martyn@imendio.com>
  */
 
 /*
@@ -62,7 +60,6 @@
 #include <libxml/tree.h>
 
 #include "gossip-chatroom-manager.h"
-#include "gossip-protocol.h"
 #include "gossip-debug.h"
 #include "gossip-session.h"
 #include "gossip-contact-manager.h"
@@ -173,8 +170,8 @@ static gboolean        log_check_dir                           (gchar
 #ifdef HAVE_GNOME
 static LogVersion      log_check_version                       (void);
 #endif
-static gchar *         log_get_basedir                         (GossipAccount         *account,
-								const gchar           *log_dir);
+static gboolean        log_check_dir                           (gchar **directory);
+static gchar *         log_get_basedir                         (GossipAccount         *account);
 static gchar *         log_get_timestamp_from_message          (GossipMessage         *msg);
 static gchar *         log_get_timestamp_filename              (void);
 static gchar *         log_get_contact_id_from_filename        (const gchar           *filename);
@@ -182,11 +179,9 @@ static GossipChatroom *log_get_chatroom_from_filename          (GossipLogManager
 								GossipAccount         *account,
 								const gchar           *filename);
 static gboolean        log_get_all_log_files                   (GList                **files);
-static void            log_get_all_log_files_for_chatroom_dir  (const gchar           *chatrooms_dir,
-								GList                **files);
 static void            log_get_all_log_files_for_chatrooms_dir (const gchar           *chatrooms_dir,
 								GList                **files);
-static void            log_get_all_log_files_for_contact_dir   (const gchar           *contact_dir,
+static void            log_get_all_log_files_in_directory      (const gchar           *directory,
 								GList                **files);
 static void            log_get_all_log_files_for_account_dir   (const gchar           *account_dir,
 								GList                **files);
@@ -201,6 +196,8 @@ static gchar *         log_get_filename_by_date_for_chatroom   (GossipChatroom
 								const gchar           *particular_date);
 static gboolean        log_set_name                            (GossipLogManager      *manager,
 								GossipContact         *contact);
+static gchar *         log_get_contact_log_dir                 (GossipContact *contact);
+static gchar *         log_get_chatroom_log_dir                (GossipChatroom *chatroom);
 
 G_DEFINE_TYPE (GossipLogManager, gossip_log_manager, G_TYPE_OBJECT);
 
@@ -404,9 +401,8 @@ log_move_contact_dirs (GossipLogManager *manager,
 		GossipAccountManager *account_manager;
 
 		account_manager = gossip_session_get_account_manager (priv->session);
-		account = gossip_account_manager_find_by_id (account_manager,
-							     basename,
-							     type_str);
+		account = gossip_account_manager_find_by_id (account_manager, basename);
+
 		if (!account) {
 			/* We must have other directories in
 			 * here which are not account
@@ -669,34 +665,6 @@ log_handlers_notify_all (GossipLogManager *manager,
 	g_list_free (handlers);
 }
 
-static gboolean
-log_check_dir (gchar **directory)
-{
-	gchar    *dir;
-	gboolean  created = FALSE;
-	gchar    *filename;
-
-	dir = g_build_filename (g_get_home_dir (), ".gnome2", PACKAGE_NAME, "logs", NULL);
-	if (!g_file_test (dir, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR)) {
-		gossip_debug (DEBUG_DOMAIN, "Creating directory:'%s'", dir);
-		created = TRUE;
-		g_mkdir_with_parents (dir, LOG_DIR_CREATE_MODE);
-
-		filename = g_build_filename (dir, "version", NULL);
-		g_file_set_contents (filename, LOG_PROTOCOL_VERSION, -1, NULL);
-		g_chmod (filename, LOG_FILE_CREATE_MODE);
-		g_free (filename);
-	}
-
-	if (directory) {
-		*directory = dir;
-	} else {
-		g_free (dir);
-	}
-
-	return created;
-}
-
 #ifdef HAVE_GNOME
 static LogVersion
 log_check_version (void)
@@ -790,20 +758,54 @@ log_check_version (void)
 }
 #endif /* HAVE_GNOME */
 
+static gboolean
+log_check_dir (gchar **directory)
+{
+	gchar    *dir;
+	gboolean  created = FALSE;
+	gchar    *filename;
+
+	dir = g_build_filename (g_get_home_dir (), ".gnome2", PACKAGE_NAME, "logs", NULL);
+	if (!g_file_test (dir, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR)) {
+		gossip_debug (DEBUG_DOMAIN, "Creating directory:'%s'", dir);
+		created = TRUE;
+		g_mkdir_with_parents (dir, LOG_DIR_CREATE_MODE);
+
+		filename = g_build_filename (dir, "version", NULL);
+		g_file_set_contents (filename, LOG_PROTOCOL_VERSION, -1, NULL);
+		g_chmod (filename, LOG_FILE_CREATE_MODE);
+		g_free (filename);
+	}
+
+	if (directory) {
+		*directory = dir;
+	} else {
+		g_free (dir);
+	}
+
+	return created;
+}
+
+
 static gchar *
-log_get_basedir (GossipAccount *account,
-		 const gchar   *log_dir)
+log_get_basedir (GossipAccount *account)
 {
 	const gchar *name;
 	gchar       *name_escaped;
 	gchar       *basedir;
+	gchar       *log_dir;
+
+	if (log_check_dir (&log_dir)) {
+		gossip_debug (DEBUG_DOMAIN, "No log directory exists");
+		g_free (log_dir);
+		return NULL;
+	}
 
 	name = gossip_account_get_name (account);
 	name_escaped = log_escape (name);
-	basedir = g_build_filename (log_dir,
-				    name_escaped,
-				    NULL);
+	basedir = g_build_filename (log_dir, name_escaped, NULL);
 	g_free (name_escaped);
+	g_free (log_dir);
 
 	return basedir;
 }
@@ -960,42 +962,6 @@ log_get_date_from_filename (const gchar *filename)
 	return g_strndup (start, end - start);
 }
 
-static void
-log_get_all_log_files_for_chatroom_dir (const gchar  *chatroom_dir,
-					 GList       **files)
-{
-	GDir        *dir;
-	const gchar *name;
-	gchar       *path;
-
-	dir = g_dir_open (chatroom_dir, 0, NULL);
-	if (!dir) {
-		gossip_debug (DEBUG_DOMAIN, "Could not open directory:'%s'", chatroom_dir);
-		return;
-	}
-
-	while ((name = g_dir_read_name (dir)) != NULL) {
-		if (!g_str_has_suffix (name, LOG_FILENAME_SUFFIX)) {
-			continue;
-		}
-
-		path = g_build_filename (chatroom_dir, name, NULL);
-
-		if (g_file_test (path, G_FILE_TEST_IS_REGULAR)) {
-			*files = g_list_insert_sorted (*files,
-						       path,
-						       (GCompareFunc) strcmp);
-
-			/* Don't free path. */
-			continue;
-		}
-
-		g_free (path);
-	}
-
-	g_dir_close (dir);
-}
-
 static void
 log_get_all_log_files_for_chatrooms_dir (const gchar  *chatrooms_dir,
 					 GList       **files)
@@ -1012,7 +978,7 @@ log_get_all_log_files_for_chatrooms_dir (const gchar  *chatrooms_dir,
 
 	while ((name = g_dir_read_name (dir)) != NULL) {
 		path = g_build_filename (chatrooms_dir, name, NULL);
-		log_get_all_log_files_for_chatroom_dir (path, files);
+		log_get_all_log_files_in_directory (path, files);
 		g_free (path);
 	}
 
@@ -1020,16 +986,15 @@ log_get_all_log_files_for_chatrooms_dir (const gchar  *chatrooms_dir,
 }
 
 static void
-log_get_all_log_files_for_contact_dir (const gchar  *contact_dir,
-				       GList       **files)
+log_get_all_log_files_in_directory (const gchar *directory, GList **files)
 {
 	GDir        *dir;
 	const gchar *name;
 	gchar       *path;
 
-	dir = g_dir_open (contact_dir, 0, NULL);
+	dir = g_dir_open (directory, 0, NULL);
 	if (!dir) {
-		gossip_debug (DEBUG_DOMAIN, "Could not open directory:'%s'", contact_dir);
+		gossip_debug (DEBUG_DOMAIN, "Could not open directory:'%s'", directory);
 		return;
 	}
 
@@ -1038,7 +1003,7 @@ log_get_all_log_files_for_contact_dir (const gchar  *contact_dir,
 			continue;
 		}
 
-		path = g_build_filename (contact_dir, name, NULL);
+		path = g_build_filename (directory, name, NULL);
 
 		if (g_file_test (path, G_FILE_TEST_IS_REGULAR)) {
 			*files = g_list_insert_sorted (*files,
@@ -1080,7 +1045,7 @@ log_get_all_log_files_for_account_dir (const gchar  *account_dir,
 		if (strcmp (name, LOG_DIR_CHATROOMS) == 0) {
 			log_get_all_log_files_for_chatrooms_dir (path, files);
 		} else {
-			log_get_all_log_files_for_contact_dir (path, files);
+			log_get_all_log_files_in_directory (path, files);
 		}
 
 		g_free (path);
@@ -1159,57 +1124,44 @@ static gchar *
 log_get_filename_by_date_for_contact (GossipContact *contact,
 				      const gchar   *particular_date)
 {
-	GossipAccount *account;
-	const gchar   *contact_id;
-	gchar         *contact_id_escaped;
-	gchar         *log_directory;
 	gchar         *filename;
 	gchar         *dirname;
+	gchar         *directory;
 	gchar         *basename;
-	gchar         *basedir;
 	gchar         *todays_date = NULL;
 	const gchar   *date;
 
+	directory = log_get_contact_log_dir (contact);
+	if (!directory) {
+		return NULL;
+	}
+
 	if (!particular_date) {
 		date = todays_date = log_get_timestamp_filename ();
 	} else {
 		date = particular_date;
 	}
 
-	account = gossip_contact_get_account (contact);
-	if (!account) {
-		gossip_debug (DEBUG_DOMAIN, "Contact has no account, can not get log filename");
-		return NULL;
-	}
-
-	log_check_dir (&log_directory);
 	basename = g_strconcat (date, LOG_FILENAME_SUFFIX, NULL);
-	basedir = log_get_basedir (account, log_directory);
 
-	contact_id = gossip_contact_get_id (contact);
-	contact_id_escaped = log_escape (contact_id);
-
-	filename = g_build_filename (basedir,
-				     contact_id_escaped,
+	filename = g_build_filename (directory,
 				     basename,
 				     NULL);
+	
+	g_free (todays_date);
+	g_free (directory);
 	g_free (basename);
-	g_free (contact_id_escaped);
-	g_free (basedir);
 
 	gossip_debug (DEBUG_DOMAIN, "Using file:'%s' for contact:'%s'",
-		    filename,
-		    gossip_contact_get_id (contact));
+		      filename,
+		      gossip_contact_get_id (contact));
 
 	dirname = g_path_get_dirname (filename);
 	if (!g_file_test (dirname, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR)) {
 		gossip_debug (DEBUG_DOMAIN, "Creating directory:'%s'", dirname);
 		g_mkdir_with_parents (dirname, LOG_DIR_CREATE_MODE);
 	}
-
 	g_free (dirname);
-	g_free (log_directory);
-	g_free (todays_date);
 
 	return filename;
 }
@@ -1218,48 +1170,35 @@ static gchar *
 log_get_filename_by_date_for_chatroom (GossipChatroom *chatroom,
 				       const gchar    *particular_date)
 {
-	GossipAccount *account;
-	const gchar   *chatroom_id;
-	gchar         *chatroom_id_escaped;
-	gchar         *log_directory;
 	gchar         *filename;
 	gchar         *dirname;
 	gchar         *basename;
-	gchar         *basedir;
+	gchar         *directory;
 	gchar         *todays_date = NULL;
 	const gchar   *date;
 
+	directory = log_get_chatroom_log_dir (chatroom);
+	if (!directory) {
+		return NULL;
+	}
+
 	if (!particular_date) {
 		date = todays_date = log_get_timestamp_filename ();
 	} else {
 		date = particular_date;
 	}
 
-	account = gossip_chatroom_get_account (chatroom);
-	if (!account) {
-		gossip_debug (DEBUG_DOMAIN, "Contact has no account, can not get log filename");
-		return NULL;
-	}
-
-	log_check_dir (&log_directory);
 	basename = g_strconcat (date, LOG_FILENAME_SUFFIX, NULL);
-	basedir = log_get_basedir (account, log_directory);
 
-	chatroom_id = gossip_chatroom_get_id_str (chatroom);
-	chatroom_id_escaped = log_escape (chatroom_id);
-	filename = g_build_filename (basedir,
-				     LOG_DIR_CHATROOMS,
-				     chatroom_id_escaped,
-				     basename,
-				     NULL);
+	filename = g_build_filename (directory, basename, NULL);
 	
 	g_free (basename);
-	g_free (chatroom_id_escaped);
-	g_free (basedir);
+	g_free (directory);
+	g_free (todays_date);
 
 	gossip_debug (DEBUG_DOMAIN, "Using file:'%s' for chatroom:'%s'",
-		    filename,
-		    gossip_chatroom_get_id_str (chatroom));
+		      filename,
+		      gossip_chatroom_get_id_str (chatroom));
 
 	dirname = g_path_get_dirname (filename);
 	if (!g_file_test (dirname, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR)) {
@@ -1268,8 +1207,6 @@ log_get_filename_by_date_for_chatroom (GossipChatroom *chatroom,
 	}
 
 	g_free (dirname);
-	g_free (log_directory);
-	g_free (todays_date);
 
 	return filename;
 }
@@ -1295,14 +1232,12 @@ log_set_name (GossipLogManager *manager,
 }
 
 GList *
-gossip_log_get_contacts (GossipLogManager *manager,
-			 GossipAccount    *account)
+gossip_log_get_contacts (GossipLogManager *manager, GossipAccount *account)
 {
 	GossipLogManagerPriv *priv;
 	GossipContactManager *contact_manager;
 	GossipContact        *contact;
 	GList                *contacts = NULL;
-	gchar                *log_directory;
 	gchar                *directory;
 	GDir                 *dir;
 	const gchar          *filename;
@@ -1313,14 +1248,7 @@ gossip_log_get_contacts (GossipLogManager *manager,
 
 	priv = GET_PRIV (manager);
 
-	if (log_check_dir (&log_directory)) {
-		gossip_debug (DEBUG_DOMAIN, "No log directory exists");
-		g_free (log_directory);
-		return NULL;
-	}
-
-	directory = log_get_basedir (account, log_directory);
-	g_free (log_directory);
+	directory = log_get_basedir (account);
 
 	dir = g_dir_open (directory, 0, NULL);
 	if (!dir) {
@@ -1364,12 +1292,10 @@ gossip_log_get_contacts (GossipLogManager *manager,
 }
 
 GList *
-gossip_log_get_chatrooms (GossipLogManager *manager,
-			  GossipAccount    *account)
+gossip_log_get_chatrooms (GossipLogManager *manager, GossipAccount *account)
 {
 	GList          *chatrooms = NULL;
 	GossipChatroom *chatroom;
-	gchar          *log_directory;
 	gchar          *directory;
 	GDir           *dir;
 	const gchar    *filename;
@@ -1379,13 +1305,7 @@ gossip_log_get_chatrooms (GossipLogManager *manager,
 	g_return_val_if_fail (GOSSIP_IS_LOG_MANAGER (manager), NULL);
 	g_return_val_if_fail (GOSSIP_IS_ACCOUNT (account), NULL);
 
-	if (log_check_dir (&log_directory)) {
-		gossip_debug (DEBUG_DOMAIN, "No log directory exists");
-		g_free (log_directory);
-		return NULL;
-	}
-
-	basedir = log_get_basedir (account, log_directory);
+	basedir = log_get_basedir (account);
 
 	directory = g_build_path (G_DIR_SEPARATOR_S,
 				  basedir,
@@ -1393,7 +1313,6 @@ gossip_log_get_chatrooms (GossipLogManager *manager,
 				  NULL);
 
 	g_free (basedir);
-	g_free (log_directory);
 
 	dir = g_dir_open (directory, 0, NULL);
 	if (!dir) {
@@ -1444,33 +1363,15 @@ gossip_log_get_date_readable (const gchar *date)
 	return gossip_time_to_string_local (t, "%a %d %b %Y");
 }
 
-/*
- * Contact functions
- */
-GList *
-gossip_log_get_dates_for_contact (GossipContact *contact)
+static gchar *
+log_get_contact_log_dir (GossipContact *contact)
 {
-	GossipAccount  *account;
-	GList          *dates = NULL;
-	gchar          *date;
-	gchar          *log_directory;
-	gchar          *directory;
-	gchar          *basedir;
-	const gchar    *contact_id;
-	gchar          *contact_id_escaped;
-	GDir           *dir;
-	const gchar    *filename;
-	const gchar    *p;
-
-	account = gossip_contact_get_account (contact);
-
-	if (log_check_dir (&log_directory)) {
-		gossip_debug (DEBUG_DOMAIN, "No log directory exists");
-		g_free (log_directory);
-		return NULL;
-	}
-
-	basedir = log_get_basedir (account, log_directory);
+	gchar       *directory;
+	const gchar *contact_id;
+	gchar       *contact_id_escaped;
+	gchar       *basedir;
+		
+	basedir = log_get_basedir (gossip_contact_get_account (contact));
 
 	contact_id = gossip_contact_get_id (contact);
 	contact_id_escaped = log_escape (contact_id);
@@ -1482,7 +1383,27 @@ gossip_log_get_dates_for_contact (GossipContact *contact)
 
 	g_free (contact_id_escaped);
 	g_free (basedir);
-	g_free (log_directory);
+
+	return directory;
+}
+
+/*
+ * Contact functions
+ */
+GList *
+gossip_log_get_dates_for_contact (GossipContact *contact)
+{
+	GList       *dates = NULL;
+	gchar       *date;
+	gchar       *directory;
+	GDir        *dir;
+	const gchar *filename;
+	const gchar *p;
+
+	directory = log_get_contact_log_dir (contact);
+	if (!directory) {
+		return NULL;
+	}
 
 	dir = g_dir_open (directory, 0, NULL);
 	if (!dir) {
@@ -1521,7 +1442,6 @@ gossip_log_get_messages_for_contact (GossipLogManager *manager,
 				     const gchar      *date)
 {
 	GossipLogManagerPriv *priv;
-	GossipAccount        *account;
 	GossipContact        *own_contact;
 	gchar                *filename;
 	GList                *messages = NULL;
@@ -1568,8 +1488,8 @@ gossip_log_get_messages_for_contact (GossipLogManager *manager,
 		return NULL;
 	}
 
-	account = gossip_contact_get_account (contact);
-	own_contact = gossip_session_get_own_contact (priv->session, account);
+	own_contact = gossip_session_get_own_contact (priv->session, 
+						      gossip_contact_get_account (contact));
 
 	if (gossip_contact_get_name (own_contact) == NULL ||
 	    strcmp (gossip_contact_get_id (own_contact),
@@ -1873,36 +1793,19 @@ gossip_log_get_last_for_contact (GossipLogManager *manager,
 	return messages;
 }
 
-/*
- * Chatroom functions
- */
-GList *
-gossip_log_get_dates_for_chatroom (GossipChatroom *chatroom)
+static gchar *
+log_get_chatroom_log_dir (GossipChatroom *chatroom)
 {
-	GossipAccount  *account;
-	GList          *dates = NULL;
-	gchar          *date;
-	gchar          *log_directory;
-	gchar          *directory;
-	gchar          *basedir;
-	const gchar    *chatroom_id;
-	gchar          *chatroom_id_escaped;
-	GDir           *dir;
-	const gchar    *filename;
-	const gchar    *p;
-
-	account = gossip_chatroom_get_account (chatroom);
-	if (!account) {
+	gchar       *directory;
+	const gchar *chatroom_id;
+	gchar       *chatroom_id_escaped;
+	gchar       *basedir;
+	
+	if (!gossip_chatroom_get_account (chatroom)) {
 		return NULL;
 	}
 
-	if (log_check_dir (&log_directory)) {
-		gossip_debug (DEBUG_DOMAIN, "No log directory exists");
-		g_free (log_directory);
-		return NULL;
-	}
-
-	basedir = log_get_basedir (account, log_directory);
+	basedir = log_get_basedir (gossip_chatroom_get_account (chatroom));
 
 	chatroom_id = gossip_chatroom_get_id_str (chatroom);
 	chatroom_id_escaped = log_escape (chatroom_id);
@@ -1912,10 +1815,26 @@ gossip_log_get_dates_for_chatroom (GossipChatroom *chatroom)
 				  LOG_DIR_CHATROOMS,
 				  chatroom_id_escaped,
 				  NULL);
-
 	g_free (basedir);
 	g_free (chatroom_id_escaped);
-	g_free (log_directory);
+
+	return directory;
+}
+
+/*
+ * Chatroom functions
+ */
+GList *
+gossip_log_get_dates_for_chatroom (GossipChatroom *chatroom)
+{
+	GList          *dates = NULL;
+	gchar          *date;
+	gchar          *directory;
+	GDir           *dir;
+	const gchar    *filename;
+	const gchar    *p;
+
+	directory = log_get_chatroom_log_dir (chatroom);
 
 	dir = g_dir_open (directory, 0, NULL);
 	if (!dir) {
@@ -1925,6 +1844,8 @@ gossip_log_get_dates_for_chatroom (GossipChatroom *chatroom)
 	}
 
 	gossip_debug (DEBUG_DOMAIN, "Collating a list of dates in:'%s'", directory);
+	
+	g_free (directory);
 
 	while ((filename = g_dir_read_name (dir)) != NULL) {
 		if (!g_str_has_suffix (filename, LOG_FILENAME_SUFFIX)) {
@@ -1940,7 +1861,6 @@ gossip_log_get_dates_for_chatroom (GossipChatroom *chatroom)
 		dates = g_list_insert_sorted (dates, date, (GCompareFunc) strcmp);
 	}
 
-	g_free (directory);
 	g_dir_close (dir);
 
 	gossip_debug (DEBUG_DOMAIN, "Parsed %d dates", g_list_length (dates));
@@ -1954,7 +1874,6 @@ gossip_log_get_messages_for_chatroom (GossipLogManager *manager,
 				      const gchar      *date)
 {
 	GossipLogManagerPriv *priv;
-	GossipAccount        *account;
 	GossipContact        *own_contact;
 	gchar                *filename;
 	GList                *messages = NULL;
@@ -1979,8 +1898,8 @@ gossip_log_get_messages_for_chatroom (GossipLogManager *manager,
 	}
 
 	/* Get own contact from log contact. */
-	account = gossip_chatroom_get_account (chatroom);
-	own_contact = gossip_session_get_own_contact (priv->session, account);
+	own_contact = gossip_session_get_own_contact (priv->session, 
+						      gossip_chatroom_get_account (chatroom));
 
 	/* Create parser. */
 	ctxt = xmlNewParserCtxt ();
@@ -2007,7 +1926,6 @@ gossip_log_get_messages_for_chatroom (GossipLogManager *manager,
 
 	/* Now get the messages. */
 	for (node = log_node->children; node; node = node->next) {
-		GossipProtocol *protocol;
 		GossipMessage  *message;
 		GossipContact  *contact;
 		gchar          *time;
@@ -2015,7 +1933,6 @@ gossip_log_get_messages_for_chatroom (GossipLogManager *manager,
 		gchar          *who;
 		gchar          *name;
 		gchar          *body;
-		gboolean        sent = FALSE;
 
 		if (strcmp (node->name, "message") != 0) {
 			continue;
@@ -2032,22 +1949,20 @@ gossip_log_get_messages_for_chatroom (GossipLogManager *manager,
 			continue;
 		}
 
- 		protocol = gossip_session_get_protocol (priv->session, account); 
-		contact = gossip_protocol_new_contact (protocol, who, name);
 		body = xmlNodeGetContent (node);
 
 		time = xmlGetProp (node, "time");
 		t = gossip_time_parse (time);
+		
+		contact = gossip_contact_new_full (GOSSIP_CONTACT_TYPE_CHATROOM,
+						   gossip_chatroom_get_account (chatroom),
+						   who, name);
 
-		sent = gossip_contact_equal (contact, own_contact);
+		message = gossip_message_new (GOSSIP_MESSAGE_TYPE_CHAT_ROOM,
+					      own_contact);
 
-		if (sent) {
-			message = gossip_message_new (GOSSIP_MESSAGE_TYPE_CHAT_ROOM, contact);
-			gossip_message_set_sender (message, own_contact);
-		} else {
-			message = gossip_message_new (GOSSIP_MESSAGE_TYPE_CHAT_ROOM, own_contact);
-			gossip_message_set_sender (message, contact);
-		}
+		gossip_message_set_sender (message, contact);
+		g_object_unref (contact);
 
 		gossip_message_set_body (message, body);
 		gossip_message_set_timestamp (message, t);
@@ -2076,7 +1991,6 @@ gossip_log_message_for_chatroom (GossipLogManager *manager,
 				 gboolean          incoming)
 {
 	GossipLogManagerPriv *priv;
-	GossipAccount        *account;
 	GossipContact        *contact;
 	GossipContact        *own_contact;
 	gchar                *filename;
@@ -2103,8 +2017,8 @@ gossip_log_message_for_chatroom (GossipLogManager *manager,
 
 	contact = gossip_message_get_sender (message);
 
-	account = gossip_chatroom_get_account (chatroom);
-	own_contact = gossip_session_get_own_contact (priv->session, account);
+	own_contact = gossip_session_get_own_contact (priv->session, 
+						      gossip_chatroom_get_account (chatroom));
 
 	filename = log_get_filename_by_date_for_chatroom (chatroom, NULL);
 
