@@ -26,11 +26,11 @@
 #include "gossip-presence.h"
 #include "gossip-account.h"
 #include "gossip-contact.h"
+#include "gossip-jabber.h"
 #include "gossip-log.h"
 #include "gossip-async.h"
 #include "gossip-chatroom-provider.h"
 #include "gossip-account-manager.h"
-#include "gossip-protocol.h"
 #include "gossip-message.h"
 #include "gossip-vcard.h"
 #include "gossip-contact-manager.h"
@@ -64,7 +64,7 @@ GossipSession * gossip_session_new                     (const gchar
 							const gchar             *chatrooms_file);
 
 /* Get protocol */
-GossipProtocol *gossip_session_get_protocol            (GossipSession           *session,
+GossipJabber *  gossip_session_get_protocol            (GossipSession           *session,
 							GossipAccount           *account);
 
 /* Providers */
@@ -92,8 +92,7 @@ void            gossip_session_count_accounts          (GossipSession          *
 							guint                  *connected,
 							guint                  *connecting,
 							guint                  *disconnected);
-GossipAccount * gossip_session_new_account             (GossipSession          *session,
-							GossipAccountType       type);
+GossipAccount * gossip_session_new_account             (GossipSession          *session);
 gboolean        gossip_session_add_account             (GossipSession          *session,
 							GossipAccount          *account);
 gboolean        gossip_session_remove_account          (GossipSession          *session,
