@@ -26,16 +26,25 @@
 
 #include <glib.h>
 
-typedef long gossip_time_t;
+G_BEGIN_DECLS
 
-gossip_time_t gossip_time_from_tm           (struct tm     *tm);
-struct tm *   gossip_time_to_tm             (gossip_time_t  t);
-gossip_time_t gossip_time_get_current       (void);
-gchar *       gossip_time_to_timestamp      (gossip_time_t  t);
-gchar *       gossip_time_to_timestamp_full (gossip_time_t  t,
-					     const gchar   *format);
-gossip_time_t gossip_time_from_string_full  (const gchar   *time,
-					     const gchar   *format);
+#define GOSSIP_TIME_FORMAT_DISPLAY_SHORT "%H:%M"
+#define GOSSIP_TIME_FORMAT_DISPLAY_LONG  "%a %d %b %Y"
+
+/* Note: Always in UTC. */
+typedef long GossipTime;
+
+GossipTime  gossip_time_get_current     (void);
+time_t      gossip_time_get_local_time  (struct tm   *tm);
+GossipTime  gossip_time_parse           (const gchar *str);
+GossipTime  gossip_time_parse_format    (const gchar *str,
+					 const gchar *format);
+gchar      *gossip_time_to_string_utc   (GossipTime   t,
+					 const gchar *format);
+gchar      *gossip_time_to_string_local (GossipTime   t,
+					 const gchar *format);
+
+G_END_DECLS
 
 #endif /* __GOSSIP_TIME_H__ */
 
