@@ -84,10 +84,7 @@ about_dialog_activate_link_cb (GtkAboutDialog *about,
 void
 gossip_about_dialog_new (GtkWindow *parent)
 {
-	gchar       *license_trans;
-	gchar       *comments;
-	gchar       *technology;
-	const gchar *backend;
+	gchar *license_trans;
 
 	gtk_about_dialog_set_url_hook (about_dialog_activate_link_cb, NULL, NULL);
 
@@ -96,34 +93,21 @@ gossip_about_dialog_new (GtkWindow *parent)
 				     _(license[2]), "\n\n",
 				     NULL);
 
-#ifdef HAVE_TELEPATHY
-	backend = "Telepathy";
-#else
-	backend = "Jabber";
-#endif
-	
-	technology = g_strdup_printf (_("Using the %s backend"), backend);
-	comments = g_strdup_printf ("%s\n%s",
-				    _("An Instant Messaging client for GNOME"),
-	                            technology);
-	g_free (technology);
-
 	gtk_show_about_dialog (parent,
 			       "artists", artists,
 			       "authors", authors,
-			       "comments", comments,
+			       "comments", _("An Instant Messaging client for GNOME"),
 			       "license", license_trans,
 			       "wrap-license", TRUE,
 			       "copyright", "Imendio AB 2002-2007",
 			       "documenters", documenters,
-			       "logo-icon-name", "gossip",
+			       "logo-icon-name", PACKAGE_TARNAME,
 			       "translator-credits", _("translator-credits"),
 			       "version", PACKAGE_VERSION,
 			       "website", WEB_SITE,
 			       NULL);
 
 	g_free (license_trans);
-	g_free (comments);
 }
 
 
