@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2005 Imendio AB
+ * Copyright (C) 2005-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,6 +16,8 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ * 
+ * Authors: Martyn Russell <martyn@imendio.com>
  */
 
 #ifndef __GOSSIP_ACCOUNT_CHOOSER_H__
@@ -47,13 +49,18 @@ struct _GossipAccountChooserClass {
 };
 
 GType          gossip_account_chooser_get_type           (void) G_GNUC_CONST;
-
 GtkWidget *    gossip_account_chooser_new                (GossipSession        *session);
+
+GossipAccount *gossip_account_chooser_get_account        (GossipAccountChooser *chooser);
+gboolean       gossip_account_chooser_get_can_select_all (GossipAccountChooser *account_chooser);
+gboolean       gossip_account_chooser_get_has_all_option (GossipAccountChooser *account_chooser);
+
 gboolean       gossip_account_chooser_set_account        (GossipAccountChooser *account_chooser,
 							  GossipAccount        *account);
-GossipAccount *gossip_account_chooser_get_account        (GossipAccountChooser *chooser);
 void           gossip_account_chooser_set_can_select_all (GossipAccountChooser *account_chooser,
 							  gboolean              can_select_all);
+void           gossip_account_chooser_set_has_all_option (GossipAccountChooser *account_chooser,
+							  gboolean              has_all_option);
 
 G_END_DECLS
 
