@@ -29,22 +29,14 @@
 #include <glib/gi18n.h>
 
 #include <libgossip/gossip-debug.h>
-#include <libgossip/gossip-protocol.h>
 #include <libgossip/gossip-session.h>
 #include <libgossip/gossip-utils.h>
 #include <libgossip/gossip-account-manager.h>
 
-#ifdef HAVE_TELEPATHY
-#include <protocols/telepathy/gossip-telepathy-cmgr.h>
-#include "gossip-protocol-chooser.h"
-#endif
-
 #include "gossip-app.h"
-#include "gossip-account-widget-irc.h"
 #include "gossip-account-widget-jabber.h"
-#include "gossip-account-widget-msn.h"
-#include "gossip-account-widget-generic.h"
 #include "gossip-accounts-dialog.h"
+#include "gossip-glade.h"
 #include "gossip-ui-utils.h"
 
 #define DEBUG_DOMAIN "AccountsDialog"
@@ -73,9 +65,7 @@ typedef struct {
 	GtkWidget *button_create;
 	GtkWidget *button_back;
 
-	GtkWidget *image_type;
 	GtkWidget *label_name;
-	GtkWidget *label_type;
 	GtkWidget *checkbutton_connect;
 	GtkWidget *checkbutton_proxy;
 	GtkWidget *settings_widget;
@@ -131,19 +121,19 @@ static gboolean       accounts_dialog_flash_connecting_foreach  (GtkTreeModel
 static gboolean       accounts_dialog_flash_connecting_cb       (GossipAccountsDialog *dialog);
 static void           accounts_dialog_protocol_connecting_cb    (GossipSession        *session,
 								 GossipAccount        *account,
-								 GossipProtocol       *protocol,
+								 GossipJabber         *jabber,
 								 GossipAccountsDialog *dialog);
 static void           accounts_dialog_protocol_connected_cb     (GossipSession        *session,
 								 GossipAccount        *account,
-								 GossipProtocol       *protocol,
+								 GossipJabber         *jabber,
 								 GossipAccountsDialog *dialog);
 static void           accounts_dialog_protocol_disconnected_cb  (GossipSession        *session,
 								 GossipAccount        *account,
-								 GossipProtocol       *protocol,
+								 GossipJabber         *jabber,
 								 gint                  reason,
 								 GossipAccountsDialog *dialog);
 static void           accounts_dialog_protocol_error_cb         (GossipSession        *session,
-								 GossipProtocol       *protocol,
+								 GossipJabber         *jabber,
 								 GossipAccount        *account,
 								 GError               *error,
 								 GossipAccountsDialog *dialog);
@@ -237,16 +227,16 @@ accounts_dialog_setup (GossipAccountsDialog *dialog)
 			      is_connected ? "YES" : "NO",
 			      is_connecting ? "YES" : "NO");
 
-		gtk_list_store_append (store, &iter);
-		gtk_list_store_set (store, &iter,
-				    COL_NAME, name,
-				    COL_EDITABLE, is_editable,
-				    COL_DEFAULT, is_default,
-				    COL_CONNECTED, is_connected,
-				    COL_CONNECTING, is_connecting,
-				    COL_AUTO_CONNECT, gossip_account_get_auto_connect (account),
-				    COL_ACCOUNT_POINTER, account,
-				    -1);
+		gtk_list_store_insert_with_values (store, &iter,
+						   -1,
+						   COL_NAME, name,
+						   COL_EDITABLE, is_editable,
+						   COL_DEFAULT, is_default,
+						   COL_CONNECTED, is_connected,
+						   COL_CONNECTING, is_connecting,
+						   COL_AUTO_CONNECT, gossip_account_get_auto_connect (account),
+						   COL_ACCOUNT_POINTER, account,
+						   -1);
 
 		g_signal_connect (account, "notify::name",
 				  G_CALLBACK (accounts_dialog_account_name_changed_cb), 
@@ -412,28 +402,9 @@ accounts_dialog_update_account (GossipAccountsDialog *dialog,
 		gtk_widget_hide (dialog->frame_no_account);
 		gtk_widget_show (dialog->vbox_details);
 
-		switch (gossip_account_get_type (account)) {
-		case GOSSIP_ACCOUNT_TYPE_IRC:
-			dialog->settings_widget = 
-				gossip_account_widget_irc_new (account);
-			break;
+		dialog->settings_widget = 
+			gossip_account_widget_jabber_new (account);
 
-		case GOSSIP_ACCOUNT_TYPE_JABBER_LEGACY:
-		case GOSSIP_ACCOUNT_TYPE_JABBER:
-			dialog->settings_widget = 
-				gossip_account_widget_jabber_new (account);
-			break;
-
-		case GOSSIP_ACCOUNT_TYPE_MSN:
-			dialog->settings_widget = 
-				gossip_account_widget_msn_new (account);
-			break;
-
-		default:
-			dialog->settings_widget = 
-				gossip_account_widget_generic_new (account);
-		}
-		
 		gtk_widget_grab_focus (dialog->settings_widget);
 	}
 
@@ -443,19 +414,9 @@ accounts_dialog_update_account (GossipAccountsDialog *dialog,
 	}
 
 	if (account) {
-		GossipAccountType  account_type;
-		const gchar       *account_type_str;
-		GdkPixbuf         *pixbuf;
+		const gchar *account_type_str;
 
-		pixbuf = gossip_pixbuf_from_account (account, GTK_ICON_SIZE_DIALOG);
-		gtk_image_set_from_pixbuf (GTK_IMAGE (dialog->image_type), pixbuf);
-		if (pixbuf) {
-			g_object_unref (pixbuf);
-		}
-
-		account_type = gossip_account_get_type (account);
-		account_type_str = gossip_account_type_to_string (account_type);
-		gtk_label_set_text (GTK_LABEL (dialog->label_type), account_type_str);
+		account_type_str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_JABBER);
 		gtk_label_set_text (GTK_LABEL (dialog->label_name), 
 				    gossip_account_get_name (account));
 
@@ -574,7 +535,7 @@ accounts_dialog_model_pixbuf_data_func (GtkTreeViewColumn    *tree_column,
 			    COL_ACCOUNT_POINTER, &account,
 			    -1);
 
-	pixbuf = gossip_pixbuf_from_account (account, GTK_ICON_SIZE_BUTTON);
+	pixbuf = gossip_account_create_pixbuf (account, GTK_ICON_SIZE_BUTTON);
 
 	if (pixbuf) {
 		if ((!is_connecting && !is_connected) ||
@@ -882,7 +843,7 @@ accounts_dialog_flash_connecting_cb (GossipAccountsDialog *dialog)
 static void
 accounts_dialog_protocol_connecting_cb (GossipSession        *session,
 					GossipAccount        *account,
-					GossipProtocol       *protocol,
+					GossipJabber         *jabber,
 					GossipAccountsDialog *dialog)
 {
 	accounts_dialog_set_connecting (dialog, account);
@@ -891,7 +852,7 @@ accounts_dialog_protocol_connecting_cb (GossipSession        *session,
 static void
 accounts_dialog_protocol_connected_cb (GossipSession        *session,
 				       GossipAccount        *account,
-				       GossipProtocol       *protocol,
+				       GossipJabber         *jabber,
 				       GossipAccountsDialog *dialog)
 {
 	GtkTreeView      *view;
@@ -956,7 +917,7 @@ accounts_dialog_protocol_connected_cb (GossipSession        *session,
 static void
 accounts_dialog_protocol_disconnected_cb (GossipSession        *session,
 					  GossipAccount        *account,
-					  GossipProtocol       *protocol,
+					  GossipJabber         *jabber,
 					  gint                  reason,
 					  GossipAccountsDialog *dialog)
 {
@@ -1021,7 +982,7 @@ accounts_dialog_protocol_disconnected_cb (GossipSession        *session,
 
 static void
 accounts_dialog_protocol_error_cb (GossipSession        *session,
-				   GossipProtocol       *protocol,
+				   GossipJabber         *jabber,
 				   GossipAccount        *account,
 				   GError               *error,
 				   GossipAccountsDialog *dialog)
@@ -1119,33 +1080,11 @@ accounts_dialog_button_create_clicked_cb (GtkWidget             *button,
 
 	account_type = GOSSIP_ACCOUNT_TYPE_UNKNOWN;
 
-	str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_JABBER_LEGACY);
-	if (strcmp (account_type_str, str) == 0) {
-		account_type = GOSSIP_ACCOUNT_TYPE_JABBER_LEGACY;
-	} 
-
-#ifdef HAVE_TELEPATHY
- 	str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_JABBER);
+	str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_JABBER);
 	if (strcmp (account_type_str, str) == 0) {
 		account_type = GOSSIP_ACCOUNT_TYPE_JABBER;
 	} 
 
- 	str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_MSN);
-	if (strcmp (account_type_str, str) == 0) {
-		account_type = GOSSIP_ACCOUNT_TYPE_MSN;
-	}
-
- 	str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_IRC);
-	if (strcmp (account_type_str, str) == 0) {
-		account_type = GOSSIP_ACCOUNT_TYPE_IRC;
-	}
-
- 	str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_SALUT);
-	if (strcmp (account_type_str, str) == 0) {
-		account_type = GOSSIP_ACCOUNT_TYPE_SALUT;
-	}
-#endif
-	
 	if (account_type == GOSSIP_ACCOUNT_TYPE_UNKNOWN) {
 		g_warning ("We have an unknown account type selected in the GtkComboBox");
 		return;
@@ -1154,7 +1093,7 @@ accounts_dialog_button_create_clicked_cb (GtkWidget             *button,
 	/* Create account */
 	session = gossip_app_get_session ();
 	manager = gossip_session_get_account_manager (session);
-	account = gossip_session_new_account (session, account_type);
+	account = gossip_session_new_account (session);
 
 	str = gtk_entry_get_text (GTK_ENTRY (dialog->entry_name));
 	gossip_account_set_name (account, str);
@@ -1468,8 +1407,6 @@ gossip_accounts_dialog_show (GossipAccount *account)
 				       "entry_name", &dialog->entry_name,
 				       "button_create", &dialog->button_create,
 				       "button_back", &dialog->button_back,
-				       "image_type", &dialog->image_type,
-				       "label_type", &dialog->label_type,
 				       "label_name", &dialog->label_name,
 				       "checkbutton_connect", &dialog->checkbutton_connect,
 				       "checkbutton_proxy", &dialog->checkbutton_proxy,
@@ -1498,23 +1435,9 @@ gossip_accounts_dialog_show (GossipAccount *account)
 	g_object_unref (glade);
 
 	/* Set up combo */
-	str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_JABBER_LEGACY);
+	str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_JABBER);
 	gtk_combo_box_append_text (GTK_COMBO_BOX (dialog->combobox_type), str);
 
-#ifdef HAVE_TELEPATHY
- 	str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_JABBER);
-	gtk_combo_box_append_text (GTK_COMBO_BOX (dialog->combobox_type), str);
-
- 	str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_MSN);
-	gtk_combo_box_append_text (GTK_COMBO_BOX (dialog->combobox_type), str);
-
- 	str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_IRC);
-	gtk_combo_box_append_text (GTK_COMBO_BOX (dialog->combobox_type), str);
-
- 	str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_SALUT);
-	gtk_combo_box_append_text (GTK_COMBO_BOX (dialog->combobox_type), str);
-#endif
-
 	/* Set up signalling */
 	session = gossip_app_get_session ();
 	manager = gossip_session_get_account_manager (session);
@@ -1548,7 +1471,6 @@ gossip_accounts_dialog_show (GossipAccount *account)
 
 	gtk_window_set_transient_for (GTK_WINDOW (dialog->window),
 				      GTK_WINDOW (gossip_app_get_window ()));
-	gtk_widget_show (dialog->window);
 
 	if (GOSSIP_IS_ACCOUNT (account)) {
 		/* If account was specified then we select it */
@@ -1556,6 +1478,8 @@ gossip_accounts_dialog_show (GossipAccount *account)
 	} else {
 		accounts_dialog_model_select_first (dialog);
 	}
+
+	gtk_widget_show (dialog->window);
 }
 
 gboolean
