@@ -35,7 +35,6 @@
 #include <libgossip/gossip-event.h>
 #include <libgossip/gossip-presence.h>
 #include <libgossip/gossip-chatroom-provider.h>
-#include <libgossip/gossip-protocol.h>
 #include <libgossip/gossip-account-manager.h>
 #include <libgossip/gossip-conf.h>
 #include <libgossip/gossip-debug.h>
@@ -50,7 +49,7 @@
 #include "gossip-contact-list.h"
 #include "gossip-glade.h"
 #include "gossip-self-presence.h"
-#include "gossip-ft-window.h"
+#include "gossip-ft-dialog.h"
 #include "gossip-geometry.h"
 #include "gossip-group-chat.h"
 #include "gossip-hint.h"
@@ -98,9 +97,6 @@
 /* Minimum width of roster window if something goes wrong. */
 #define MIN_WIDTH 50
 
-/* Accels (menu shortcuts) can be configured and saved */
-#define ACCELS_FILENAME "accels.txt"
-
 /* Delay for retry to connect when disconnected (seconds) */
 #define RETRY_CONNECT_TIMEOUT 20
 
@@ -246,24 +242,24 @@ app_throbber_button_press_event_cb           (GtkWidget             *throbber,
 static void     
 app_session_protocol_connecting_cb           (GossipSession         *session,
 					      GossipAccount         *account,
-					      GossipProtocol        *protocol,
+					      GossipJabber          *jabber,
 					      gpointer               user_data);
 static void     
 app_session_protocol_connected_cb            (GossipSession         *session,
 					      GossipAccount         *account,
-					      GossipProtocol        *protocol,
+					      GossipJabber          *jabber,
 					      gpointer               user_data);
 static void     app_reconnect_remove         (gpointer               p);
 static gboolean app_reconnect_cb             (GossipAccount         *account);
 static void     
 app_session_protocol_disconnected_cb         (GossipSession         *session,
 					      GossipAccount         *account,
-					      GossipProtocol        *protocol,
+					      GossipJabber          *jabber,
 					      gint                   reason,
 					      gpointer               user_data);
 static void     
 app_session_protocol_error_cb                (GossipSession         *session,
-					      GossipProtocol        *protocol,
+					      GossipJabber          *jabber,
 					      GossipAccount         *account,
 					      GError                *error,
 					      gpointer               user_data);
@@ -447,7 +443,7 @@ app_finalize (GObject *object)
 					      app_event_added_cb,
 					      NULL);
 
-	gossip_ft_window_finalize (priv->session);
+	gossip_ft_dialog_finalize (priv->session);
 	gossip_subscription_dialog_finalize (priv->session);
 	
 	g_object_unref (priv->self_presence);
@@ -661,6 +657,8 @@ app_setup_contact_list_sort_criterium (void)
 				      "sort-criterium", enum_value->value,
 				      NULL);
 		}
+
+		g_free (str);
 	} 
 }
 
@@ -711,7 +709,7 @@ app_setup (GossipSession *session)
 		      "Initialising session listeners "
 		      "(subscription, file transfer, etc)");
 	gossip_subscription_dialog_init (priv->session);
-	gossip_ft_window_init (priv->session);
+	gossip_ft_dialog_init (priv->session);
 
 	if (gossip_accounts_dialog_is_needed ()) {
 		gossip_debug (DEBUG_DOMAIN_SETUP,
@@ -1116,7 +1114,7 @@ app_favorite_chatroom_menu_update (void)
 		account = gossip_chatroom_get_account (chatroom);
 		menu_item = g_object_get_data (G_OBJECT (chatroom), "menu_item");
 
-		visible = gossip_chatroom_get_is_favourite (chatroom);
+		visible = gossip_chatroom_get_favourite (chatroom);
 		visible &= gossip_session_is_connected (priv->session, account);
 
 		if (visible) {
@@ -1152,7 +1150,7 @@ app_favorite_chatroom_menu_add (GossipChatroom *chatroom)
 	account = gossip_chatroom_get_account (chatroom);
 	menu_item = g_object_get_data (G_OBJECT (chatroom), "menu_item");
 
-	visible = gossip_chatroom_get_is_favourite (chatroom);
+	visible = gossip_chatroom_get_favourite (chatroom);
 	visible &= gossip_session_is_connected (priv->session, account);
 
 	if (menu_item) {
@@ -1215,7 +1213,7 @@ app_favorite_chatroom_menu_removed_cb (GossipChatroomManager *manager,
 	account = gossip_chatroom_get_account (chatroom);
 	menu_item = g_object_get_data (G_OBJECT (chatroom), "menu_item");
 
-	visible = gossip_chatroom_get_is_favourite (chatroom);
+	visible = gossip_chatroom_get_favourite (chatroom);
 	visible &= gossip_session_is_connected (priv->session, account);
 
 	g_object_set_data (G_OBJECT (chatroom), "menu_item", NULL);
@@ -1512,7 +1510,7 @@ app_throbber_button_press_event_cb (GtkWidget      *throbber_ebox,
 static void
 app_session_protocol_connecting_cb (GossipSession  *session,
 				    GossipAccount  *account,
-				    GossipProtocol *protocol,
+				    GossipJabber   *jabber,
 				    gpointer        user_data)
 {
 	GossipAppPriv *priv;
@@ -1540,10 +1538,10 @@ app_restore_saved_presence (void)
 }
 
 static void
-app_session_protocol_connected_cb (GossipSession  *session,
-				   GossipAccount  *account,
-				   GossipProtocol *protocol,
-				   gpointer        user_data)
+app_session_protocol_connected_cb (GossipSession *session,
+				   GossipAccount *account,
+				   GossipJabber  *jabber,
+				   gpointer       user_data)
 {
 	GossipAppPriv *priv;
 	gboolean       connecting;
@@ -1595,11 +1593,11 @@ app_reconnect_cb (GossipAccount *account)
 }
 
 static void
-app_session_protocol_disconnected_cb (GossipSession  *session,
-				      GossipAccount  *account,
-				      GossipProtocol *protocol,
-				      gint            reason,
-				      gpointer        user_data)
+app_session_protocol_disconnected_cb (GossipSession *session,
+				      GossipAccount *account,
+				      GossipJabber  *jabber,
+				      gint           reason,
+				      gpointer       user_data)
 {
 	GossipAppPriv *priv;
 	gboolean       connecting;
@@ -1627,7 +1625,7 @@ app_session_protocol_disconnected_cb (GossipSession  *session,
 	app_favorite_chatroom_menu_update ();
 	gossip_self_presence_updated (gossip_app_get_self_presence ());
 
-	should_reconnect = reason != GOSSIP_PROTOCOL_DISCONNECT_ASKED;
+	should_reconnect = reason != GOSSIP_JABBER_DISCONNECT_ASKED;
 
 #ifdef HAVE_DBUS
 	/* If NM says we are offline that's useless to retry to connect,
@@ -1655,7 +1653,7 @@ app_session_protocol_disconnected_cb (GossipSession  *session,
 
 static void
 app_session_protocol_error_cb (GossipSession  *session,
-			       GossipProtocol *protocol,
+			       GossipJabber   *jabber,
 			       GossipAccount  *account,
 			       GError         *error,
 			       gpointer        user_data)
@@ -1706,7 +1704,7 @@ app_restore_user_accels (void)
 	gchar *filename;
 
 	gossip_debug (DEBUG_DOMAIN_SETUP, "Configuring accels");
-	filename = g_build_filename (g_get_home_dir (), ".gnome2", PACKAGE_NAME, ACCELS_FILENAME, NULL);
+	filename = g_build_filename (g_get_home_dir (), ".gnome2", "accels", PACKAGE_TARNAME, NULL);
 	if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
 		gossip_debug (DEBUG_DOMAIN_ACCELS, "Loading from:'%s'", filename);
 		gtk_accel_map_load (filename);
@@ -1721,9 +1719,9 @@ app_save_user_accels (void)
 	gchar *dir;
 	gchar *file_with_path;
 
-	dir = g_build_filename (g_get_home_dir (), ".gnome2", PACKAGE_NAME, NULL);
+	dir = g_build_filename (g_get_home_dir (), ".gnome2", "accels", NULL);
 	g_mkdir_with_parents (dir, S_IRUSR | S_IWUSR | S_IXUSR);
-	file_with_path = g_build_filename (dir, ACCELS_FILENAME, NULL);
+	file_with_path = g_build_filename (dir, PACKAGE_TARNAME, NULL);
 	g_free (dir);
 
 	gossip_debug (DEBUG_DOMAIN_ACCELS, "Saving to:'%s'", file_with_path);
@@ -2035,6 +2033,8 @@ app_notify_sort_criterium_cb (GossipConf  *conf,
 			gossip_contact_list_set_sort_criterium (priv->contact_list, 
 								enum_value->value);
 		}
+		
+		g_free (str);
 	}
 }
 
