@@ -16,11 +16,6 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
- * 
- * Authors: Mikael Hallendal <micke@imendio.com>
- *          Richard Hult <richard@imendio.com>
- *          Martyn Russell <martyn@imendio.com>
- *          Geert-Jan Van den Bogaerde <geertjan@gnome.org>
  */
 
 #include "config.h"
@@ -38,18 +33,21 @@
 #include <libgossip/gossip-message.h>
 #include <libgossip/gossip-chatroom-manager.h>
 #include <libgossip/gossip-conf.h>
+#include <libgossip/gossip-stock.h>
 
 #include "gossip-add-contact-dialog.h"
 #include "gossip-app.h"
 #include "gossip-chat-invite.h"
 #include "gossip-chat-window.h"
 #include "gossip-contact-info-dialog.h"
+#include "gossip-email.h"
+#include "gossip-glade.h"
 #include "gossip-log-window.h"
 #include "gossip-new-chatroom-dialog.h"
 #include "gossip-preferences.h"
 #include "gossip-private-chat.h"
+#include "gossip-smiley.h"
 #include "gossip-sound.h"
-#include "gossip-stock.h"
 #include "gossip-ui-utils.h"
 
 #define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_CHAT_WINDOW, GossipChatWindowPriv))
@@ -82,10 +80,12 @@ struct _GossipChatWindowPriv {
 	GtkWidget   *menu_conv_separator;
 	GtkWidget   *menu_conv_add_contact;
 	GtkWidget   *menu_conv_info;
+	GtkWidget   *menu_conv_email;
 	GtkWidget   *menu_conv_close;
 
 	GtkWidget   *menu_room;
 	GtkWidget   *menu_room_set_topic;
+	GtkWidget   *menu_room_set_nick;
 	GtkWidget   *menu_room_join_new;
 	GtkWidget   *menu_room_invite;
 	GtkWidget   *menu_room_add;
@@ -104,123 +104,127 @@ struct _GossipChatWindowPriv {
 	guint        save_geometry_id;
 };
 
-static void       gossip_chat_window_class_init         (GossipChatWindowClass *klass);
-static void       gossip_chat_window_init               (GossipChatWindow      *window);
-static void       gossip_chat_window_finalize           (GObject               *object);
-static GdkPixbuf *chat_window_get_status_pixbuf         (GossipChatWindow      *window,
-							 GossipChat            *chat);
-static void       chat_window_accel_cb                  (GtkAccelGroup         *accelgroup,
-							 GObject               *object,
-							 guint                  key,
-							 GdkModifierType        mod,
-							 GossipChatWindow      *window);
-static void       chat_window_avatar_changed_cb         (GossipContact         *contact,
-							 GParamSpec            *param,
-							 GossipChatWindow      *window);
-static void       chat_window_close_clicked_cb          (GtkWidget             *button,
-							 GossipChat            *chat);
-static GtkWidget *chat_window_create_label              (GossipChatWindow      *window,
-							 GossipChat            *chat);
-static void       chat_window_update_status             (GossipChatWindow      *window,
-							 GossipChat            *chat);
-static void       chat_window_update_title              (GossipChatWindow      *window,
-							 GossipChat            *chat);
-static void       chat_window_update_menu               (GossipChatWindow      *window);
-static gboolean   chat_window_save_geometry_timeout_cb  (GossipChatWindow      *window);
-static gboolean   chat_window_configure_event_cb        (GtkWidget             *widget,
-							 GdkEventConfigure     *event,
-							 GossipChatWindow      *window);
-static void       chat_window_conv_activate_cb          (GtkWidget             *menuitem,
-							 GossipChatWindow      *window);
-static void       chat_window_clear_activate_cb         (GtkWidget             *menuitem,
-							 GossipChatWindow      *window);
-static void       chat_window_info_activate_cb          (GtkWidget             *menuitem,
-							 GossipChatWindow      *window);
-static void       chat_window_add_contact_activate_cb   (GtkWidget             *menuitem,
-							 GossipChatWindow      *window);
-static void       chat_window_log_activate_cb           (GtkWidget             *menuitem,
-							 GossipChatWindow      *window);
-static void       chat_window_show_contacts_toggled_cb  (GtkWidget             *menuitem,
-							 GossipChatWindow      *window);
-static void       chat_window_edit_activate_cb          (GtkWidget             *menuitem,
-							 GossipChatWindow      *window);
-static void       chat_window_insert_smiley_activate_cb (GtkWidget             *menuitem,
-							 GossipChatWindow      *window);
-static void       chat_window_close_activate_cb         (GtkWidget             *menuitem,
-							 GossipChatWindow      *window);
-static void       chat_window_room_set_topic_activate_cb(GtkWidget             *menuitem,
-							 GossipChatWindow      *window);
-static void       chat_window_room_join_new_activate_cb (GtkWidget             *menuitem,
-							 GossipChatWindow      *window);
-static void       chat_window_room_invite_activate_cb   (GtkWidget             *menuitem,
-							 GossipChatWindow      *window);
-static void       chat_window_room_add_activate_cb      (GtkWidget             *menuitem,
-							 GossipChatWindow      *window);
-static void       chat_window_cut_activate_cb           (GtkWidget             *menuitem,
-							 GossipChatWindow      *window);
-static void       chat_window_copy_activate_cb          (GtkWidget             *menuitem,
-							 GossipChatWindow      *window);
-static void       chat_window_paste_activate_cb         (GtkWidget             *menuitem,
-							 GossipChatWindow      *window);
-static void       chat_window_tabs_left_activate_cb     (GtkWidget             *menuitem,
-							 GossipChatWindow      *window);
-static void       chat_window_tabs_right_activate_cb    (GtkWidget             *menuitem,
-							 GossipChatWindow      *window);
-static void       chat_window_detach_activate_cb        (GtkWidget             *menuitem,
-							 GossipChatWindow      *window);
-static gboolean   chat_window_delete_event_cb           (GtkWidget             *dialog,
-							 GdkEvent              *event,
-							 GossipChatWindow      *window);
-static void       chat_window_status_changed_cb         (GossipChat            *chat,
-							 GossipChatWindow      *window);
-static void       chat_window_update_tooltip            (GossipChatWindow      *window,
-							 GossipChat            *chat);
-static void       chat_window_name_changed_cb           (GossipChat            *chat,
-							 const gchar           *name,
-							 GossipChatWindow      *window);
-static void       chat_window_composing_cb              (GossipChat            *chat,
-							 gboolean               is_composing,
-							 GossipChatWindow      *window);
-static void       chat_window_new_message_cb            (GossipChat            *chat,
-							 GossipMessage         *message,
-							 GossipChatWindow      *window);
-static void       chat_window_disconnected_cb           (GossipApp             *app,
-							 GossipChatWindow      *window);
-static GtkNotebook* chat_window_detach_hook             (GtkNotebook           *source,
-							 GtkWidget             *page,
-							 gint                   x,
-							 gint                   y,
-							 gpointer               user_data);
-static void       chat_window_page_switched_cb          (GtkNotebook           *notebook,
-							 GtkNotebookPage       *page,
-							 gint                   page_num,
-							 GossipChatWindow      *window);
-static void       chat_window_page_reordered_cb         (GtkNotebook           *notebook,
-							 GtkWidget             *widget,
-							 guint                  page_num,
-							 GossipChatWindow      *window);
-static void       chat_window_page_added_cb             (GtkNotebook           *notebook,
-							 GtkWidget             *child,
-							 guint                  page_num,
-							 GossipChatWindow      *window);
-static void       chat_window_page_removed_cb           (GtkNotebook           *notebook,
-							 GtkWidget             *child,
-							 guint                  page_num,
-							 GossipChatWindow      *window);
-static gboolean   chat_window_focus_in_event_cb         (GtkWidget             *widget,
-							 GdkEvent              *event,
-							 GossipChatWindow      *window);
-static void       chat_window_drag_data_received        (GtkWidget             *widget,
-							 GdkDragContext        *context,
-							 int                    x,
-							 int                    y,
-							 GtkSelectionData      *selection,
-							 guint                  info,
-							 guint                  time,
-							 GossipChatWindow      *window);
-static void       chat_window_set_urgency_hint          (GossipChatWindow      *window,
-							 gboolean               urgent);
-
+static void         gossip_chat_window_class_init          (GossipChatWindowClass *klass);
+static void         gossip_chat_window_init                (GossipChatWindow      *window);
+static void         gossip_chat_window_finalize            (GObject               *object);
+static GdkPixbuf *  chat_window_get_status_pixbuf          (GossipChatWindow      *window,
+							    GossipChat            *chat);
+static void         chat_window_accel_cb                   (GtkAccelGroup         *accelgroup,
+							    GObject               *object,
+							    guint                  key,
+							    GdkModifierType        mod,
+							    GossipChatWindow      *window);
+static void         chat_window_avatar_changed_cb          (GossipContact         *contact,
+							    GParamSpec            *param,
+							    GossipChatWindow      *window);
+static void         chat_window_close_clicked_cb           (GtkWidget             *button,
+							    GossipChat            *chat);
+static GtkWidget *  chat_window_create_label               (GossipChatWindow      *window,
+							    GossipChat            *chat);
+static void         chat_window_update_status              (GossipChatWindow      *window,
+							    GossipChat            *chat);
+static void         chat_window_update_title               (GossipChatWindow      *window,
+							    GossipChat            *chat);
+static void         chat_window_update_menu                (GossipChatWindow      *window);
+static gboolean     chat_window_save_geometry_timeout_cb   (GossipChatWindow      *window);
+static gboolean     chat_window_configure_event_cb         (GtkWidget             *widget,
+							    GdkEventConfigure     *event,
+							    GossipChatWindow      *window);
+static void         chat_window_conv_activate_cb           (GtkWidget             *menuitem,
+							    GossipChatWindow      *window);
+static void         chat_window_clear_activate_cb          (GtkWidget             *menuitem,
+							    GossipChatWindow      *window);
+static void         chat_window_info_activate_cb           (GtkWidget             *menuitem,
+							    GossipChatWindow      *window);
+static void         chat_window_email_activate_cb          (GtkWidget             *menuitem,
+							    GossipChatWindow      *window);
+static void         chat_window_add_contact_activate_cb    (GtkWidget             *menuitem,
+							    GossipChatWindow      *window);
+static void         chat_window_log_activate_cb            (GtkWidget             *menuitem,
+							    GossipChatWindow      *window);
+static void         chat_window_show_contacts_toggled_cb   (GtkWidget             *menuitem,
+							    GossipChatWindow      *window);
+static void         chat_window_edit_activate_cb           (GtkWidget             *menuitem,
+							    GossipChatWindow      *window);
+static void         chat_window_insert_smiley_activate_cb  (GtkWidget             *menuitem,
+							    GossipChatWindow      *window);
+static void         chat_window_close_activate_cb          (GtkWidget             *menuitem,
+							    GossipChatWindow      *window);
+static void         chat_window_room_set_topic_activate_cb (GtkWidget             *menuitem,
+							    GossipChatWindow      *window);
+static void         chat_window_room_set_nick_activate_cb  (GtkWidget             *menuitem,
+							    GossipChatWindow      *window);
+static void         chat_window_room_join_new_activate_cb  (GtkWidget             *menuitem,
+							    GossipChatWindow      *window);
+static void         chat_window_room_invite_activate_cb    (GtkWidget             *menuitem,
+							    GossipChatWindow      *window);
+static void         chat_window_room_add_activate_cb       (GtkWidget             *menuitem,
+							    GossipChatWindow      *window);
+static void         chat_window_cut_activate_cb            (GtkWidget             *menuitem,
+							    GossipChatWindow      *window);
+static void         chat_window_copy_activate_cb           (GtkWidget             *menuitem,
+							    GossipChatWindow      *window);
+static void         chat_window_paste_activate_cb          (GtkWidget             *menuitem,
+							    GossipChatWindow      *window);
+static void         chat_window_tabs_left_activate_cb      (GtkWidget             *menuitem,
+							    GossipChatWindow      *window);
+static void         chat_window_tabs_right_activate_cb     (GtkWidget             *menuitem,
+							    GossipChatWindow      *window);
+static void         chat_window_detach_activate_cb         (GtkWidget             *menuitem,
+							    GossipChatWindow      *window);
+static gboolean     chat_window_delete_event_cb            (GtkWidget             *dialog,
+							    GdkEvent              *event,
+							    GossipChatWindow      *window);
+static void         chat_window_status_changed_cb          (GossipChat            *chat,
+							    GossipChatWindow      *window);
+static void         chat_window_update_tooltip             (GossipChatWindow      *window,
+							    GossipChat            *chat);
+static void         chat_window_name_changed_cb            (GossipChat            *chat,
+							    const gchar           *name,
+							    GossipChatWindow      *window);
+static void         chat_window_composing_cb               (GossipChat            *chat,
+							    gboolean               is_composing,
+							    GossipChatWindow      *window);
+static void         chat_window_new_message_cb             (GossipChat            *chat,
+							    GossipMessage         *message,
+							    gboolean               is_backlog,
+							    GossipChatWindow      *window);
+static void         chat_window_disconnected_cb            (GossipApp             *app,
+							    GossipChatWindow      *window);
+static GtkNotebook* chat_window_detach_hook                (GtkNotebook           *source,
+							    GtkWidget             *page,
+							    gint                   x,
+							    gint                   y,
+							    gpointer               user_data);
+static void         chat_window_page_switched_cb           (GtkNotebook           *notebook,
+							    GtkNotebookPage       *page,
+							    gint                   page_num,
+							    GossipChatWindow      *window);
+static void         chat_window_page_reordered_cb          (GtkNotebook           *notebook,
+							    GtkWidget             *widget,
+							    guint                  page_num,
+							    GossipChatWindow      *window);
+static void         chat_window_page_added_cb              (GtkNotebook           *notebook,
+							    GtkWidget             *child,
+							    guint                  page_num,
+							    GossipChatWindow      *window);
+static void         chat_window_page_removed_cb            (GtkNotebook           *notebook,
+							    GtkWidget             *child,
+							    guint                  page_num,
+							    GossipChatWindow      *window);
+static gboolean     chat_window_focus_in_event_cb          (GtkWidget             *widget,
+							    GdkEvent              *event,
+							    GossipChatWindow      *window);
+static void         chat_window_drag_data_received         (GtkWidget             *widget,
+							    GdkDragContext        *context,
+							    int                    x,
+							    int                    y,
+							    GtkSelectionData      *selection,
+							    guint                  info,
+							    guint                  time,
+							    GossipChatWindow      *window);
+static void         chat_window_set_urgency_hint           (GossipChatWindow      *window,
+							    gboolean               urgent);
 
 static GList *chat_windows = NULL;
 
@@ -293,9 +297,11 @@ gossip_chat_window_init (GossipChatWindow *window)
 				       "menu_conv_separator", &priv->menu_conv_separator,
 				       "menu_conv_add_contact", &priv->menu_conv_add_contact,
 				       "menu_conv_info", &priv->menu_conv_info,
+				       "menu_conv_email", &priv->menu_conv_email,
 				       "menu_conv_close", &priv->menu_conv_close,
 				       "menu_room", &priv->menu_room,
 				       "menu_room_set_topic", &priv->menu_room_set_topic,
+				       "menu_room_set_nick", &priv->menu_room_set_nick,
 				       "menu_room_join_new", &priv->menu_room_join_new,
 				       "menu_room_invite", &priv->menu_room_invite,
 				       "menu_room_add", &priv->menu_room_add,
@@ -318,8 +324,10 @@ gossip_chat_window_init (GossipChatWindow *window)
 			      "menu_conv_log", "activate", chat_window_log_activate_cb,
 			      "menu_conv_add_contact", "activate", chat_window_add_contact_activate_cb,
 			      "menu_conv_info", "activate", chat_window_info_activate_cb,
+			      "menu_conv_email", "activate", chat_window_email_activate_cb,
 			      "menu_conv_close", "activate", chat_window_close_activate_cb,
 			      "menu_room_set_topic", "activate", chat_window_room_set_topic_activate_cb,
+			      "menu_room_set_nick", "activate", chat_window_room_set_nick_activate_cb,
 			      "menu_room_join_new", "activate", chat_window_room_join_new_activate_cb,
 			      "menu_room_invite", "activate", chat_window_room_invite_activate_cb,
 			      "menu_room_add", "activate", chat_window_room_add_activate_cb,
@@ -524,12 +532,14 @@ chat_window_get_status_pixbuf (GossipChatWindow *window,
 	priv = GET_PRIV (window);
 
 	if (g_list_find (priv->chats_new_msg, chat)) {
-		pixbuf = gossip_pixbuf_from_stock (GOSSIP_STOCK_MESSAGE,
-						   GTK_ICON_SIZE_MENU);
+		pixbuf = gossip_stock_create_pixbuf (gossip_app_get_window (),
+						     GOSSIP_STOCK_MESSAGE,
+						     GTK_ICON_SIZE_MENU);
 	}
 	else if (g_list_find (priv->chats_composing, chat)) {
-		pixbuf = gossip_pixbuf_from_stock (GOSSIP_STOCK_TYPING,
-						   GTK_ICON_SIZE_MENU);
+		pixbuf = gossip_stock_create_pixbuf (gossip_app_get_window (),
+						     GOSSIP_STOCK_TYPING,
+						     GTK_ICON_SIZE_MENU);
 	}
 	else {
 		pixbuf = gossip_chat_get_status_pixbuf (chat);
@@ -770,8 +780,9 @@ chat_window_update_title (GossipChatWindow *window,
 	g_free (title);
 
 	if (priv->chats_new_msg) {
-		pixbuf = gossip_pixbuf_from_stock (GOSSIP_STOCK_MESSAGE,
-						   GTK_ICON_SIZE_MENU);
+		pixbuf = gossip_stock_create_pixbuf (gossip_app_get_window (),
+						     GOSSIP_STOCK_MESSAGE,
+						     GTK_ICON_SIZE_MENU);
 	} else {
 		pixbuf = NULL;
 	}
@@ -977,6 +988,19 @@ chat_window_info_activate_cb (GtkWidget        *menuitem,
 					 GTK_WINDOW (priv->dialog));
 }
 
+static void
+chat_window_email_activate_cb (GtkWidget *menuitem, GossipChatWindow *window)
+{
+	GossipChatWindowPriv *priv;
+	GossipContact        *contact;
+
+	priv = GET_PRIV (window);
+
+	contact = gossip_chat_get_contact (priv->current_chat);
+
+	gossip_email_open (contact);
+}
+
 static gboolean
 chat_window_save_geometry_timeout_cb (GossipChatWindow *window)
 {
@@ -1039,12 +1063,17 @@ chat_window_conv_activate_cb (GtkWidget        *menuitem,
 		if (chatroom) {
 			log_exists = gossip_log_exists_for_chatroom (chatroom);
 		}
+
+		gtk_widget_hide (priv->menu_conv_email);
 	} else {
 		GossipContact *contact;
 
 		contact = gossip_chat_get_contact (priv->current_chat);
 		if (contact) {
 			log_exists = gossip_log_exists_for_contact (contact);
+			gtk_widget_show (priv->menu_conv_email);
+			gtk_widget_set_sensitive (priv->menu_conv_email, 
+						  gossip_email_available (contact));
 		}
 	}
 
@@ -1095,6 +1124,22 @@ chat_window_room_set_topic_activate_cb (GtkWidget        *menuitem,
 	}
 }
 
+static void
+chat_window_room_set_nick_activate_cb (GtkWidget        *menuitem,
+				       GossipChatWindow *window)
+{
+	GossipChatWindowPriv *priv;
+	
+	priv = GET_PRIV (window);
+
+	if (gossip_chat_is_group_chat (priv->current_chat)) {
+		GossipGroupChat *group_chat;
+
+		group_chat = GOSSIP_GROUP_CHAT (priv->current_chat);
+		gossip_group_chat_set_nick (group_chat);
+	}
+}
+
 static void
 chat_window_room_join_new_activate_cb (GtkWidget        *menuitem,
 				       GossipChatWindow *window)
@@ -1392,6 +1437,7 @@ chat_window_composing_cb (GossipChat       *chat,
 static void
 chat_window_new_message_cb (GossipChat       *chat,
 			    GossipMessage    *message,
+			    gboolean          is_backlog,
 			    GossipChatWindow *window)
 {
 	GossipChatWindowPriv *priv;
@@ -1414,7 +1460,8 @@ chat_window_new_message_cb (GossipChat       *chat,
 	if (gossip_chat_is_group_chat (chat)) {
 		own_contact = gossip_chat_get_own_contact (chat);
 		
-		if (gossip_chat_should_highlight_nick (message, own_contact)) {
+		if (!is_backlog && 
+		    gossip_chat_should_highlight_nick (message, own_contact)) {
 			gossip_debug (DEBUG_DOMAIN, "Highlight this nick");
 			needs_urgency = TRUE;
 		}
@@ -1427,7 +1474,8 @@ chat_window_new_message_cb (GossipChat       *chat,
 		gossip_request_user_attention ();
 	}
 
-	if (!g_list_find (priv->chats_new_msg, chat)) {
+	if (!is_backlog && 
+	    !g_list_find (priv->chats_new_msg, chat)) {
 		priv->chats_new_msg = g_list_prepend (priv->chats_new_msg, chat);
 		chat_window_update_status (window, chat);
 	}
