@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2003-2006 Imendio AB
+ * Copyright (C) 2003-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,19 +18,25 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
+
 #include <string.h>
+
 #include <glade/glade.h>
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 
-#include <libgossip/gossip-contact.h>
+#include <libgossip/gossip-account.h>
+#include <libgossip/gossip-version-info.h>
+#include <libgossip/gossip-presence.h>
 #include <libgossip/gossip-session.h>
-#include <libgossip/gossip-paths.h>
 #include <libgossip/gossip-utils.h>
+#include <libgossip/gossip-vcard.h>
+#include <libgossip/gossip-paths.h>
 
 #include "gossip-app.h"
 #include "gossip-avatar-image.h"
+#include "gossip-glade.h"
 #include "gossip-ui-utils.h"
 #include "gossip-contact-info-dialog.h"
 
@@ -207,6 +213,7 @@ contact_info_dialog_update_presences (GossipContactInfoDialog *dialog)
 		widget = gtk_label_new (status);
 		gtk_label_set_line_wrap (GTK_LABEL (widget), TRUE);
 		gtk_misc_set_alignment (GTK_MISC (widget), 0, 0.5);
+		gtk_label_set_selectable(GTK_LABEL (widget), TRUE);
 		gtk_table_attach (GTK_TABLE (dialog->presence_table),
 				  widget,
 				  1, 2,
@@ -215,7 +222,7 @@ contact_info_dialog_update_presences (GossipContactInfoDialog *dialog)
 				  GTK_FILL,
 				  0, 0);
 
-		if (!resource) {
+		if (G_STR_EMPTY (resource)) {
 			continue;
 		}
 
@@ -268,7 +275,7 @@ contact_info_dialog_get_vcard_cb (GossipResult   result,
 		return;
 	}
 
-	pixbuf = gossip_pixbuf_avatar_from_vcard (vcard);
+	pixbuf = gossip_vcard_create_avatar_pixbuf (vcard);
 	if (pixbuf != NULL) {
 		gossip_avatar_image_set_pixbuf (GOSSIP_AVATAR_IMAGE (dialog->avatar_image), pixbuf);
 		g_object_unref (pixbuf);
