@@ -31,7 +31,6 @@
 
 #include <libgossip/gossip-contact.h>
 #include <libgossip/gossip-session.h>
-#include <libgossip/gossip-protocol.h>
 #include <libgossip/gossip-utils.h>
 #include <libgossip/gossip-account.h>
 #include <libgossip/gossip-log.h>
@@ -117,16 +116,16 @@ static void            group_chat_retry_connection_clicked_cb (GtkWidget
 							       GossipGroupChat              *chat);
 static void            group_chat_join                        (GossipGroupChat              *chat);
 static void            group_chat_join_cb                     (GossipChatroomProvider       *provider,
-							       GossipChatroomJoinResult      result,
 							       GossipChatroomId              id,
+							       GossipChatroomError           error,
 							       gpointer                      user_data);
 static void            group_chat_protocol_connected_cb       (GossipSession                *session,
 							       GossipAccount                *account,
-							       GossipProtocol               *protocol,
+							       GossipJabber                 *jabber,
 							       GossipGroupChat              *chat);
 static void            group_chat_protocol_disconnected_cb    (GossipSession                *session,
 							       GossipAccount                *account,
-							       GossipProtocol               *protocol,
+							       GossipJabber                 *jabber,
 							       gint                          reason,
 							       GossipGroupChat              *chat);
 static gboolean        group_chat_key_press_event_cb          (GtkWidget                    *widget,
@@ -178,11 +177,19 @@ static void            group_chat_topic_changed_cb            (GossipChatroomPro
 							       GossipContact                *who,
 							       const gchar                  *new_topic,
 							       GossipGroupChat              *chat);
-static void            group_chat_topic_entry_activate_cb     (GtkWidget                    *entry,
+static void            group_chat_error_cb                    (GossipChatroomProvider       *provider,
+							       GossipChatroomId              id,
+							       GossipChatroomError           error,
+							       GossipGroupChat              *chat);
+static void            group_chat_dialog_entry_activate_cb    (GtkWidget                    *entry,
 							       GtkDialog                    *dialog);
-static void            group_chat_topic_response_cb           (GtkWidget                    *dialog,
+static void            group_chat_dialog_response_cb          (GtkWidget                    *dialog,
 							       gint                          response,
 							       GossipGroupChat              *chat);
+static void            group_chat_dialog_new                  (GossipGroupChat              *chat,
+							       const gchar                  *action,
+							       const gchar                  *message,
+							       const gchar                  *pretext);
 static void            group_chat_contact_joined_cb           (GossipChatroom               *chatroom,
 							       GossipContact                *contact,
 							       GossipGroupChat              *chat);
@@ -246,6 +253,10 @@ static void            group_chat_cl_expander_cell_data_func  (GtkTreeViewColumn
 static void            group_chat_cl_set_background           (GossipGroupChat              *chat,
 							       GtkCellRenderer              *cell,
 							       gboolean                      is_header);
+static void            group_chat_set_scrolling_for_events    (GossipGroupChat              *chat,
+							       gboolean                      disable);
+static gboolean        group_chat_scroll_down_when_idle_func  (GossipGroupChat              *chat);
+static void            group_chat_scroll_down_when_idle       (GossipGroupChat              *chat);
 
 enum {
 	COL_STATUS,
@@ -374,6 +385,9 @@ group_chat_finalize (GObject *object)
 	g_signal_handlers_disconnect_by_func (priv->chatroom_provider,
 					      group_chat_topic_changed_cb,
 					      chat);
+	g_signal_handlers_disconnect_by_func (priv->chatroom_provider,
+					      group_chat_error_cb,
+					      chat);
 	g_signal_handlers_disconnect_by_func (priv->chatroom,
 					      group_chat_contact_joined_cb,
 					      chat);
@@ -413,6 +427,34 @@ group_chat_retry_connection_clicked_cb (GtkWidget       *button,
 	gtk_widget_set_sensitive (button, FALSE);
 }
 
+static void
+group_chat_set_scrolling_for_events (GossipGroupChat *chat,
+				     gboolean         disable)
+{
+	GossipGroupChatPriv *priv;
+	gboolean             recently_joined;
+
+	priv = GET_PRIV (chat);
+
+	recently_joined = priv->time_joined + 5 > gossip_time_get_current ();
+
+	if (!recently_joined) {
+		return;
+	}
+
+	if (disable) {
+		/* If we joined the group chat in the last 5 seconds, don't
+		 * scroll on new presence messages. We need to do this because
+		 * unlike normal messages, we don't have a timestamp all the
+		 * time.
+		 */
+		gossip_chat_view_allow_scroll (GOSSIP_CHAT (chat)->view, FALSE);
+	} else {
+		/* Enable scrolling again */
+		gossip_chat_view_allow_scroll (GOSSIP_CHAT (chat)->view, TRUE);
+	}
+}
+
 static void
 group_chat_join (GossipGroupChat *chat)
 {
@@ -425,17 +467,19 @@ group_chat_join (GossipGroupChat *chat)
 				       (GossipChatroomJoinCb) group_chat_join_cb,
 				       NULL);
 
+	group_chat_set_scrolling_for_events (chat, TRUE);
 	gossip_chat_view_append_event (GOSSIP_CHAT (chat)->view, 
 				       _("Connecting..."));
+	group_chat_set_scrolling_for_events (chat, FALSE);
 
 	g_signal_emit_by_name (chat, "status-changed");
 }
 
 static void     
-group_chat_join_cb (GossipChatroomProvider   *provider,
-		    GossipChatroomJoinResult  result,
-		    GossipChatroomId          id,
-		    gpointer                  user_data)
+group_chat_join_cb (GossipChatroomProvider *provider,
+		    GossipChatroomId        id,
+		    GossipChatroomError     error,
+		    gpointer                user_data)
 {
 	GossipGroupChatPriv *priv;
 	GossipChat          *chat;
@@ -444,7 +488,7 @@ group_chat_join_cb (GossipChatroomProvider   *provider,
 	GtkTreeModel        *model;
 	GtkTreeStore        *store;
 	GtkWidget           *button;
-	const gchar         *result_str;
+	const gchar         *error_str;
 
 	chat = g_hash_table_lookup (group_chats, GINT_TO_POINTER (id));
 	if (!chat) {
@@ -453,30 +497,42 @@ group_chat_join_cb (GossipChatroomProvider   *provider,
 
 	priv = GET_PRIV (chat);
 
-	result_str = gossip_chatroom_provider_join_result_as_str (result);
+	error_str = gossip_chatroom_provider_error_to_string (error);
 
 	gossip_debug (DEBUG_DOMAIN, 
-		      "Join callback for id:%d, result:%d->'%s'", 
-		      id, result, result_str);
+		      "Join callback for id:%d, error:%d->'%s'", 
+		      id, error, error_str);
 
 	chatview = chat->view;
 	g_signal_emit_by_name (chat, "status-changed");
 
-	/* Check the result */
-	switch (result) {
-	case GOSSIP_CHATROOM_JOIN_OK:
-	case GOSSIP_CHATROOM_JOIN_ALREADY_OPEN:
+	/* Set the time we joined this chatroom so we know for new
+	 * messages if they are backlog or not.
+	 */
+	priv->time_joined = gossip_time_get_current ();
+
+	/* Check the error */
+	switch (error) {
+	case GOSSIP_CHATROOM_ERROR_NONE:
+	case GOSSIP_CHATROOM_ERROR_ALREADY_OPEN:
 		break;
 
-	case GOSSIP_CHATROOM_JOIN_NEED_PASSWORD:
-	case GOSSIP_CHATROOM_JOIN_UNKNOWN_HOST:
-		gossip_chat_view_append_event (chatview, result_str);
+	case GOSSIP_CHATROOM_ERROR_PASSWORD_INVALID_OR_MISSING:
+	case GOSSIP_CHATROOM_ERROR_USER_BANNED:
+	case GOSSIP_CHATROOM_ERROR_ROOM_NOT_FOUND:
+	case GOSSIP_CHATROOM_ERROR_ROOM_CREATION_RESTRICTED:
+	case GOSSIP_CHATROOM_ERROR_USE_RESERVED_ROOM_NICK:
+	case GOSSIP_CHATROOM_ERROR_NOT_ON_MEMBERS_LIST:
+		group_chat_set_scrolling_for_events (GOSSIP_GROUP_CHAT (chat), TRUE);
+		gossip_chat_view_append_event (chatview, error_str);
+		group_chat_set_scrolling_for_events (GOSSIP_GROUP_CHAT (chat), FALSE);
 		return;
 
-	case GOSSIP_CHATROOM_JOIN_NICK_IN_USE:
-	case GOSSIP_CHATROOM_JOIN_TIMED_OUT:
-	case GOSSIP_CHATROOM_JOIN_UNKNOWN_ERROR:
-	case GOSSIP_CHATROOM_JOIN_CANCELED:
+	case GOSSIP_CHATROOM_ERROR_NICK_IN_USE:
+	case GOSSIP_CHATROOM_ERROR_MAXIMUM_USERS_REACHED:
+	case GOSSIP_CHATROOM_ERROR_TIMED_OUT:
+	case GOSSIP_CHATROOM_ERROR_UNKNOWN:
+	case GOSSIP_CHATROOM_ERROR_CANCELED:
 		/* FIXME: Need special case for nickname to put an
 		 * entry in the chat view and to request a new nick.
 		 */
@@ -485,11 +541,13 @@ group_chat_join_cb (GossipChatroomProvider   *provider,
 				  G_CALLBACK (group_chat_retry_connection_clicked_cb),
 				  chat);
 
-		gossip_chat_view_append_event (chatview, result_str);
+		group_chat_set_scrolling_for_events (GOSSIP_GROUP_CHAT (chat), TRUE);
+		gossip_chat_view_append_event (chatview, error_str);
 		gossip_chat_view_append_button (chatview,
 						NULL,
 						button,
 						NULL);
+		group_chat_set_scrolling_for_events (GOSSIP_GROUP_CHAT (chat), FALSE);
 
 		return;
 	}
@@ -505,7 +563,9 @@ group_chat_join_cb (GossipChatroomProvider   *provider,
 	gtk_widget_set_sensitive (priv->scrolled_window_contacts, TRUE);
 	gtk_widget_set_sensitive (priv->scrolled_window_input, TRUE);
 
+	group_chat_set_scrolling_for_events (GOSSIP_GROUP_CHAT (chat), TRUE);
 	gossip_chat_view_append_event (chatview, _("Connected"));
+	group_chat_set_scrolling_for_events (GOSSIP_GROUP_CHAT (chat), FALSE);
 
 	gtk_widget_grab_focus (GOSSIP_CHAT (chat)->input_text_view);
 }
@@ -513,7 +573,7 @@ group_chat_join_cb (GossipChatroomProvider   *provider,
 static void
 group_chat_protocol_connected_cb (GossipSession   *session,
 				  GossipAccount   *account,
-				  GossipProtocol  *protocol,
+				  GossipJabber    *jabber,
 				  GossipGroupChat *chat)
 {
 	GossipGroupChatPriv *priv;
@@ -538,7 +598,7 @@ group_chat_protocol_connected_cb (GossipSession   *session,
 static void
 group_chat_protocol_disconnected_cb (GossipSession   *session,
 				     GossipAccount   *account,
-				     GossipProtocol  *protocol,
+				     GossipJabber    *jabber,
 				     gint             reason,
 				     GossipGroupChat *chat)
 {
@@ -557,8 +617,10 @@ group_chat_protocol_disconnected_cb (GossipSession   *session,
 	gtk_widget_set_sensitive (priv->scrolled_window_input, FALSE);
 
 	/* i18n: Disconnected as in "was disconnected". */
+	group_chat_set_scrolling_for_events (chat, TRUE);
 	gossip_chat_view_append_event (GOSSIP_CHAT (chat)->view,
 				       _("Disconnected"));
+	group_chat_set_scrolling_for_events (chat, FALSE);
 
 	g_signal_emit_by_name (chat, "status-changed");
 }
@@ -762,7 +824,11 @@ group_chat_drag_data_received (GtkWidget        *widget,
 		/* Send event to chat window */
 		str = g_strdup_printf (_("Invited %s to join this chat conference."),
 				       gossip_contact_get_id (contact));
+
+		group_chat_set_scrolling_for_events (chat, TRUE);
 		gossip_chat_view_append_event (GOSSIP_CHAT (chat)->view, str);
+		group_chat_set_scrolling_for_events (chat, FALSE);
+
 		g_free (str);
 		
 		gossip_chat_invite_dialog_show (contact, gossip_chatroom_get_id (priv->chatroom));
@@ -1005,8 +1071,8 @@ group_chat_contacts_find (GossipGroupChat *chat,
 
 static gint
 group_chat_contacts_completion_func (const gchar *s1,
-				    const gchar *s2,
-				    gsize        n)
+				     const gchar *s2,
+				     gsize        n)
 {
 	gchar *tmp, *nick1, *nick2;
 	gint   ret;
@@ -1031,31 +1097,99 @@ static void
 group_chat_create_ui (GossipGroupChat *chat)
 {
 	GossipGroupChatPriv *priv;
-	GladeXML            *glade;
  	GList               *list = NULL; 
+	GtkWidget           *temp_widget;
 
 	priv = GET_PRIV (chat);
 
-	glade = gossip_glade_get_file ("group-chat.glade",
-				       "group_chat_widget",
-				       NULL,
-				       "group_chat_widget", &priv->widget,
-				       "hpaned", &priv->hpaned,
-				       "vbox_left", &priv->vbox_left,
-				       "scrolled_window_chat", &priv->scrolled_window_chat,
-				       "scrolled_window_input", &priv->scrolled_window_input,
-				       "hbox_topic", &priv->hbox_topic,
-				       "label_topic", &priv->label_topic,
-				       "scrolled_window_contacts", &priv->scrolled_window_contacts,
-				       "treeview", &priv->treeview,
-				       NULL);
+	priv->widget = g_object_new (GTK_TYPE_VBOX,
+				     "homogeneous", FALSE,
+				     "spacing", 6,
+				     "border-width", 4,
+				     NULL);
 
-	gossip_glade_connect (glade,
-			      chat,
-			      "group_chat_widget", "destroy", group_chat_widget_destroy_cb,
+	priv->hbox_topic = gtk_hbox_new (FALSE, 6);
+
+	gtk_box_pack_start (GTK_BOX (priv->widget), priv->hbox_topic,
+			    FALSE, FALSE, 2);
+
+	temp_widget = g_object_new (GTK_TYPE_LABEL,
+				    "label", _("<b>Topic:</b>"),
+				    "use-markup", TRUE,
+				    NULL);
+
+	gtk_box_pack_start (GTK_BOX (priv->hbox_topic), temp_widget,
+			    FALSE, FALSE, 0);
+
+	priv->label_topic = g_object_new (GTK_TYPE_LABEL,
+					  "use-markup", TRUE,
+					  "single-line-mode", TRUE,
+					  "ellipsize", PANGO_ELLIPSIZE_END,
+					  "selectable", TRUE,
+					  "xalign", 0.0,
+					  NULL);
+
+	gtk_box_pack_start (GTK_BOX (priv->hbox_topic), priv->label_topic,
+			    TRUE, TRUE, 0);
+
+	priv->hpaned = gtk_hpaned_new ();
+
+	gtk_box_pack_start (GTK_BOX (priv->widget), priv->hpaned,
+			    TRUE, TRUE, 0);
+
+	
+	priv->vbox_left = gtk_vbox_new (FALSE, 6);
+
+	gtk_paned_pack1 (GTK_PANED (priv->hpaned), priv->vbox_left,
+			 TRUE, TRUE);
+	
+	priv->scrolled_window_contacts = 
+		g_object_new (GTK_TYPE_SCROLLED_WINDOW,
+			      "hscrollbar-policy", GTK_POLICY_NEVER,
+			      "vscrollbar-policy", GTK_POLICY_AUTOMATIC,
+			      "shadow-type", GTK_SHADOW_IN,
 			      NULL);
 
-	g_object_unref (glade);
+	gtk_paned_pack2 (GTK_PANED (priv->hpaned), 
+			 priv->scrolled_window_contacts,
+			 FALSE, FALSE);
+
+	priv->scrolled_window_chat =
+		g_object_new (GTK_TYPE_SCROLLED_WINDOW,
+			      "hscrollbar-policy", GTK_POLICY_AUTOMATIC,
+			      "vscrollbar-policy", GTK_POLICY_ALWAYS,
+			      "shadow-type", GTK_SHADOW_IN,
+			      NULL);
+
+	gtk_box_pack_start (GTK_BOX (priv->vbox_left), 
+			    priv->scrolled_window_chat,
+			    TRUE, TRUE, 0);
+
+	priv->scrolled_window_input =
+		g_object_new (GTK_TYPE_SCROLLED_WINDOW,
+			      "hscrollbar-policy", GTK_POLICY_NEVER,
+			      "vscrollbar-policy", GTK_POLICY_NEVER,
+			      "shadow-type", GTK_SHADOW_IN,
+			      NULL);
+
+	gtk_box_pack_start (GTK_BOX (priv->vbox_left),
+			    priv->scrolled_window_input,
+			    FALSE, TRUE, 0);
+
+	priv->treeview = g_object_new (GTK_TYPE_TREE_VIEW,
+				       "headers-visible", FALSE,
+				       "enable-search", TRUE,
+				       "width-request", 115,
+				       NULL);
+
+	gtk_container_add (GTK_CONTAINER (priv->scrolled_window_contacts),
+			   priv->treeview);
+
+	gtk_widget_show_all (priv->widget);
+
+	g_signal_connect (priv->widget, "destroy", 
+			  G_CALLBACK (group_chat_widget_destroy_cb),
+			  chat);
 
 	g_object_set_data (G_OBJECT (priv->widget), "chat", chat);
 
@@ -1114,6 +1248,39 @@ group_chat_create_ui (GossipGroupChat *chat)
 	g_list_free (list);
 }
 
+/* Scroll down after the back-log has been received. */
+static gboolean
+group_chat_scroll_down_when_idle_func (GossipGroupChat *chat)
+{
+	GossipGroupChatPriv *priv;
+
+	priv = GET_PRIV (chat);
+
+	gossip_chat_scroll_down (GOSSIP_CHAT (chat));
+
+	priv->scroll_idle_id = 0;
+
+	return FALSE;
+}
+
+static void
+group_chat_scroll_down_when_idle (GossipGroupChat *chat)
+{
+	GossipGroupChatPriv *priv;
+	
+	priv = GET_PRIV (chat);
+	
+	/* Add an idle timeout to scroll to the bottom */
+	if (priv->scroll_idle_id) {
+		g_source_remove (priv->scroll_idle_id);
+		priv->scroll_idle_id = 0;
+	}
+	
+	priv->scroll_idle_id = g_idle_add ((GSourceFunc) 
+					   group_chat_scroll_down_when_idle_func, 
+					   chat);
+}
+
 static void
 group_chat_new_message_cb (GossipChatroomProvider *provider,
 			   gint                    id,
@@ -1139,6 +1306,10 @@ group_chat_new_message_cb (GossipChatroomProvider *provider,
 		      "[%d] New message with timestamp:%d, message %s backlog", 
 		      id, timestamp, is_backlog ? "IS" : "IS NOT");
 
+	if (is_backlog) {
+		gossip_chat_view_allow_scroll (GOSSIP_CHAT (chat)->view, FALSE);
+	}
+
 	invite = gossip_message_get_invite (message);
 	if (invite) {
 		gossip_chat_view_append_invite (GOSSIP_CHAT (chat)->view,
@@ -1162,17 +1333,25 @@ group_chat_new_message_cb (GossipChatroomProvider *provider,
 		}
 	}
 
+	/* Play sound? */
 	if (!is_backlog && 
 	    gossip_chat_should_play_sound (GOSSIP_CHAT (chat)) &&
 	    gossip_chat_should_highlight_nick (message, priv->own_contact)) {
 		gossip_sound_play (GOSSIP_SOUND_CHAT);
 	}
 
+	/* Log message? */
 	if (!is_backlog) {
 		log_manager = gossip_session_get_log_manager (gossip_app_get_session ());
 		gossip_log_message_for_chatroom (log_manager, priv->chatroom, message, FALSE);
 	}
 
+	/* Re-enable scrolling? */
+	if (is_backlog) {
+		gossip_chat_view_allow_scroll (GOSSIP_CHAT (chat)->view, TRUE);
+		group_chat_scroll_down_when_idle (chat);
+	}
+
 	g_signal_emit_by_name (chat, "new-message", message, is_backlog);
 }
 
@@ -1192,7 +1371,9 @@ group_chat_new_event_cb (GossipChatroomProvider *provider,
 
 	gossip_debug (DEBUG_DOMAIN, "[%d] New event:'%s'", id, event);
 
+	group_chat_set_scrolling_for_events (chat, TRUE);
 	gossip_chat_view_append_event (GOSSIP_CHAT (chat)->view, event);
+	group_chat_set_scrolling_for_events (chat, FALSE);
 }
 
 static void
@@ -1222,47 +1403,93 @@ group_chat_topic_changed_cb (GossipChatroomProvider *provider,
 	event = g_strdup_printf (_("%s has set the topic: %s"),
 				 gossip_contact_get_name (who),
 				 new_topic);
+
+	group_chat_set_scrolling_for_events (chat, TRUE);
 	gossip_chat_view_append_event (GOSSIP_CHAT (chat)->view, event);
+	group_chat_set_scrolling_for_events (chat, FALSE);
+
 	g_free (event);
 
 	g_signal_emit_by_name (chat, "status-changed");
 }
 
+static void     
+group_chat_error_cb (GossipChatroomProvider *provider,
+		     GossipChatroomId        id,
+		     GossipChatroomError     error,
+		     GossipGroupChat        *chat)
+{
+	GossipGroupChatPriv *priv;
+	const gchar         *error_str;
+
+	priv = GET_PRIV (chat);
+
+	if (id != gossip_chatroom_get_id (priv->chatroom)) {
+		return;
+	}
+
+	error_str = gossip_chatroom_provider_error_to_string (error);
+
+	gossip_debug (DEBUG_DOMAIN, "[%d] Error:%d->'%s'", 
+		      id, error, error_str);
+
+	group_chat_set_scrolling_for_events (GOSSIP_GROUP_CHAT (chat), TRUE);
+	gossip_chat_view_append_event (GOSSIP_CHAT (chat)->view, error_str);
+	group_chat_set_scrolling_for_events (GOSSIP_GROUP_CHAT (chat), FALSE);
+}
+
 static void
-group_chat_topic_entry_activate_cb (GtkWidget *entry,
-				    GtkDialog *dialog)
+group_chat_dialog_entry_activate_cb (GtkWidget *entry,
+				     GtkDialog *dialog)
 {
 	gtk_dialog_response (dialog, GTK_RESPONSE_OK);
 }
 
 static void
-group_chat_topic_response_cb (GtkWidget       *dialog,
-			      gint             response,			      
-			      GossipGroupChat *chat)
+group_chat_dialog_response_cb (GtkWidget       *dialog,
+			       gint             response,			      
+			       GossipGroupChat *chat)
 {
 	if (response == GTK_RESPONSE_OK) {
 		GtkWidget   *entry;
-		const gchar *topic;
+		const gchar *str;
 
 		entry = g_object_get_data (G_OBJECT (dialog), "entry");
-		topic = gtk_entry_get_text (GTK_ENTRY (entry));
+		str = gtk_entry_get_text (GTK_ENTRY (entry));
 		
-		if (!G_STR_EMPTY (topic)) {
-			GossipGroupChatPriv *priv;
+		if (!G_STR_EMPTY (str)) {
+			const gchar *purpose;
+	
+			purpose = g_object_get_data (G_OBJECT (dialog), "action");
+			if (purpose) { 
+				GossipGroupChatPriv *priv;
 
-			priv = GET_PRIV (chat);
+				priv = GET_PRIV (chat);
 
-			gossip_chatroom_provider_change_topic (priv->chatroom_provider,
-							       gossip_chatroom_get_id (priv->chatroom),
-							       topic);
+				if (strcmp (purpose, "change-topic") == 0) {
+					gossip_chatroom_provider_change_topic (priv->chatroom_provider,
+									       gossip_chatroom_get_id (priv->chatroom),
+									       str);
+				}
+				else if (strcmp (purpose, "change-nick") == 0) {
+					gossip_chatroom_provider_change_nick (priv->chatroom_provider,
+									      gossip_chatroom_get_id (priv->chatroom),
+									      str);
+				}
+			} else {
+				g_warning ("Action for group chat dialog unknown, doing nothing");
+			}
 		}
 	}
 
 	gtk_widget_destroy (dialog);
 }
 
-void
-gossip_group_chat_set_topic (GossipGroupChat *chat)
+static void
+group_chat_dialog_new (GossipGroupChat *chat,
+		       const gchar     *action,
+		       const gchar     *message, 
+		       const gchar     *pretext)
 {
 	GossipGroupChatPriv *priv;
 	GossipChatWindow    *chat_window;
@@ -1270,8 +1497,9 @@ gossip_group_chat_set_topic (GossipGroupChat *chat)
 	GtkWidget           *dialog;
 	GtkWidget           *entry;
 	GtkWidget           *hbox;
-	const gchar         *topic;
 
+	g_return_if_fail (!G_STR_EMPTY (action));
+	g_return_if_fail (!G_STR_EMPTY (message));
 
 	priv = GET_PRIV (chat);
 
@@ -1282,33 +1510,59 @@ gossip_group_chat_set_topic (GossipGroupChat *chat)
 					 0,
 					 GTK_MESSAGE_QUESTION,
 					 GTK_BUTTONS_OK_CANCEL,
-					 _("Enter the new topic you want to set for this room:"));
-
-	topic = gtk_label_get_text (GTK_LABEL (priv->label_topic));
+					 message);
 
 	hbox = gtk_hbox_new (FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
 			    hbox, FALSE, TRUE, 4);
 
 	entry = gtk_entry_new ();
-	gtk_entry_set_text (GTK_ENTRY (entry), topic);
+	gtk_entry_set_text (GTK_ENTRY (entry), pretext ? pretext : ""); 
 	gtk_editable_select_region (GTK_EDITABLE (entry), 0, -1);
 		    
 	gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 4);
 
 	g_object_set (GTK_MESSAGE_DIALOG (dialog)->label, "use-markup", TRUE, NULL);
 	g_object_set_data (G_OBJECT (dialog), "entry", entry);
+	g_object_set_data_full (G_OBJECT (dialog), "action", g_strdup (action), g_free);
 
 	g_signal_connect (entry, "activate",
-			  G_CALLBACK (group_chat_topic_entry_activate_cb),
+			  G_CALLBACK (group_chat_dialog_entry_activate_cb),
 			  dialog);
 	g_signal_connect (dialog, "response",
-			  G_CALLBACK (group_chat_topic_response_cb),
+			  G_CALLBACK (group_chat_dialog_response_cb),
 			  chat);
 
 	gtk_widget_show_all (dialog);
 }
 
+void
+gossip_group_chat_set_topic (GossipGroupChat *group_chat)
+{
+	GossipGroupChatPriv *priv;
+
+	priv = GET_PRIV (group_chat);
+
+	group_chat_dialog_new (group_chat, 
+			       "change-topic", 
+			       _("Enter the new topic you want to set for this room:"),
+			       gtk_label_get_text (GTK_LABEL (priv->label_topic)));
+
+}
+
+void
+gossip_group_chat_set_nick (GossipGroupChat *group_chat)
+{
+	GossipGroupChatPriv *priv;
+
+	priv = GET_PRIV (group_chat);
+
+	group_chat_dialog_new (group_chat, 
+			       "change-nick", 
+			       _("Enter the new nickname you want to be know by:"),
+			       gossip_chatroom_get_nick (priv->chatroom));
+}
+
 static void
 group_chat_cl_pixbuf_cell_data_func (GtkTreeViewColumn *tree_column,
 				     GtkCellRenderer   *cell,
@@ -1445,7 +1699,7 @@ group_chat_contact_joined_cb (GossipChatroom  *chatroom,
 			      GossipContact   *contact,
 			      GossipGroupChat *chat)
 {
-	GossipGroupChatPriv       *priv;
+	GossipGroupChatPriv *priv;
 
 	priv = GET_PRIV (chat);
 
@@ -1462,18 +1716,20 @@ group_chat_contact_joined_cb (GossipChatroom  *chatroom,
 			  chat);
 
 	g_signal_emit_by_name (chat, "contact_added", contact);
-	
+
 	/* Add event to chatroom */
 	if (!gossip_contact_equal (priv->own_contact, contact)) {
 		gchar *str;
 
 		str = g_strdup_printf (_("%s has joined the room"),
 				       gossip_contact_get_name (contact));
+		
+		group_chat_set_scrolling_for_events (chat, TRUE);
 		gossip_chat_view_append_event (GOSSIP_CHAT (chat)->view, str);
+		group_chat_set_scrolling_for_events (chat, FALSE);
+
 		g_free (str);
 	}
-
-	priv->time_joined = gossip_time_get_current ();
 }
 
 static void
@@ -1504,7 +1760,11 @@ group_chat_contact_left_cb (GossipChatroom  *chatroom,
 		gchar *str;
 		str = g_strdup_printf (_("%s has left the room"),
 				       gossip_contact_get_name (contact));
+
+		group_chat_set_scrolling_for_events (chat, TRUE);
 		gossip_chat_view_append_event (GOSSIP_CHAT (chat)->view, str);
+		group_chat_set_scrolling_for_events (chat, FALSE);
+
 		g_free (str);
 	}
 }
@@ -2031,22 +2291,6 @@ group_chat_get_role_iter (GossipGroupChat    *chat,
 	}
 }
 
-/* Scroll down after the back-log has been received. */
-static gboolean
-group_chat_scroll_down_idle_func (GossipChat *chat)
-{
-	GossipGroupChatPriv *priv;
-
-	priv = GET_PRIV (chat);
-
-	gossip_chat_scroll_down (chat);
-	g_object_unref (chat);
-
-	priv->scroll_idle_id = 0;
-
-	return FALSE;
-}
-
 /* Copied from the jabber backend for now since we don't have an abstraction for
  * "contacts" for group chats. Casefolds the node part (the part before @).
  */
@@ -2148,6 +2392,9 @@ gossip_group_chat_new (GossipChatroomProvider *provider,
 	g_signal_connect (provider, "chatroom-topic-changed",
 			  G_CALLBACK (group_chat_topic_changed_cb),
 			  chat);
+	g_signal_connect (provider, "chatroom-error",
+			  G_CALLBACK (group_chat_error_cb),
+			  chat);
 	g_signal_connect (chatroom, "contact-joined",
 			  G_CALLBACK (group_chat_contact_joined_cb),
 			  chat);
@@ -2171,9 +2418,7 @@ gossip_group_chat_new (GossipChatroomProvider *provider,
 	
  	gossip_chat_present (GOSSIP_CHAT (chat)); 
 
-	priv->scroll_idle_id = g_idle_add ((GSourceFunc) 
-					   group_chat_scroll_down_idle_func, 
-					   g_object_ref (chat));
+	group_chat_scroll_down_when_idle (chat);
 
 	return chat;
 }
