@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2002-2004 Imendio AB
+ * Copyright (C) 2002-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,11 +16,16 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ *
+ * Authors: Mikael Hallendal <micke@imendio.com>
+ *          Richard Hult <richard@imendio.com>
+ *          Martyn Russell <martyn@imendio.com>
  */
 
 #ifndef __GOSSIP_GROUP_CHAT_H__
 #define __GOSSIP_GROUP_CHAT_H__
 
+#include <libgossip/gossip-chatroom.h>
 #include <libgossip/gossip-chatroom-provider.h>
 
 #include "gossip-chat.h"
@@ -56,6 +61,7 @@ GossipChatroomProvider *
 		 gossip_group_chat_get_chatroom_provider (GossipGroupChat        *group_chat);
 GossipChatroom * gossip_group_chat_get_chatroom          (GossipGroupChat        *group_chat);
 void             gossip_group_chat_set_topic             (GossipGroupChat        *group_chat);
+void             gossip_group_chat_set_nick              (GossipGroupChat        *group_chat);
 
 G_END_DECLS
 
