@@ -32,18 +32,17 @@
 #include <libgossip/gossip-account.h>
 #include <libgossip/gossip-contact.h>
 #include <libgossip/gossip-event.h>
-#include <libgossip/gossip-protocol.h>
 #include <libgossip/gossip-message.h>
 #include <libgossip/gossip-presence.h>
 #include <libgossip/gossip-debug.h>
 #include <libgossip/gossip-conf.h>
+#include <libgossip/gossip-stock.h>
 
 #include "gossip-chat-manager.h"
 #include "gossip-app.h"
 #include "gossip-contact-info-dialog.h"
 #include "gossip-notify.h"
 #include "gossip-preferences.h"
-#include "gossip-stock.h"
 #include "gossip-ui-utils.h"
 
 #define DEBUG_DOMAIN "Notify"
@@ -79,11 +78,11 @@ static NotifyNotification *notify_new_message                     (GossipEventMa
 static gboolean            notify_protocol_timeout_cb             (GossipAccount      *account);
 static void                notify_protocol_connected_cb           (GossipSession      *session,
 								   GossipAccount      *account,
-								   GossipProtocol     *protocol,
+								   GossipJabber       *jabber,
 								   gpointer            user_data);
 static void                notify_protocol_disconnected_cb        (GossipSession      *session,
 								   GossipAccount      *account,
-								   GossipProtocol     *protocol,
+								   GossipJabber       *jabber,
 								   gint                reason,
 								   gpointer            user_data);
 static void                notify_contact_presence_updated_cb     (GossipContact      *contact,
@@ -217,10 +216,12 @@ notify_contact_online (GossipContact *contact)
 			       &show_avatars);
 
 	if (show_avatars) {
-		pixbuf = gossip_pixbuf_avatar_from_contact_scaled (contact, 32, 32);
+		pixbuf = gossip_contact_get_avatar_pixbuf (contact);
 	}
 
-	if (!pixbuf) {
+	if (pixbuf) {
+		g_object_ref (pixbuf);
+	} else {
 		/* Fall back to presence state */
 		pixbuf = gossip_pixbuf_for_presence (presence);
 	}
@@ -273,18 +274,46 @@ notify_subscription_request_default_cb (NotifyNotification *notify,
 static void
 notify_subscription_request_show (GossipContact *contact)
 {
-	NotifyNotification  *notify;
-	GdkPixbuf           *pixbuf = NULL;
-	GError              *error = NULL;
-	const gchar         *name;
-	gchar               *message;
-	gboolean             show_avatars = FALSE;
+	GossipSession        *session;
+	GossipAccountManager *account_manager;
+	const gchar          *contact_name;
+	guint                 n;
+	NotifyNotification   *notify;
+	GdkPixbuf            *pixbuf = NULL;
+	GError               *error = NULL;
+	gchar                *message;
+	gboolean              show_avatars = FALSE;
 
-	name = gossip_contact_get_name (contact);
-	if (name) {
-		message = g_strdup_printf (_("%s wants to be added to your contact list."), name);
-	} else {
-		message = g_strdup (_("Someone wants to be added to your contact list."));
+	session = gossip_app_get_session ();
+	account_manager = gossip_session_get_account_manager (session);
+	n = gossip_account_manager_get_count (account_manager);
+	contact_name = gossip_contact_get_name (contact);
+
+	if (n > 1) {
+		GossipAccount *account;
+		const gchar   *account_name;
+
+		account = gossip_contact_get_account (contact);
+		account_name = gossip_account_get_name (account);
+
+		if (contact_name) {
+			message = g_strdup_printf (
+				_("%s wants to be added to your contact list for your '%s' account."), 
+				contact_name, account_name);
+		} else {
+			message = g_strdup_printf (
+				_("Someone wants to be added to your contact list for your '%s' account."),
+				account_name);
+		}
+	} else { 
+		if (contact_name) {
+			message = g_strdup_printf (
+				_("%s wants to be added to your contact list."), 
+				contact_name);
+		} else {
+			message = g_strdup_printf (
+				_("Someone wants to be added to your contact list."));
+		}
 	}
 
 	gossip_conf_get_bool (gossip_conf_get (),
@@ -292,13 +321,16 @@ notify_subscription_request_show (GossipContact *contact)
 			      &show_avatars);
 
 	if (show_avatars) {
-		pixbuf = gossip_pixbuf_avatar_from_contact_scaled (contact, 32, 32);
+		pixbuf = gossip_contact_get_avatar_pixbuf (contact);
 	}
 
-	if (!pixbuf) {
+	if (pixbuf) {
+		g_object_ref (pixbuf);
+	} else {
 		/* Fall back to presence state */
-		pixbuf = gossip_pixbuf_from_stock (GTK_STOCK_DIALOG_QUESTION,
-						   GTK_ICON_SIZE_MENU);
+		pixbuf = gossip_stock_create_pixbuf (gossip_app_get_window (),
+						     GTK_STOCK_DIALOG_QUESTION,
+						     GTK_ICON_SIZE_MENU);
 	}
 
 	notify = notify_notification_new (_("Subscription request"),
@@ -307,6 +339,10 @@ notify_subscription_request_show (GossipContact *contact)
 					  NULL);
 	g_free (message);
 
+	if (attach_status_icon) {
+		notify_notification_attach_to_status_icon (notify, attach_status_icon);
+	}
+
 	notify_notification_set_urgency (notify, NOTIFY_URGENCY_NORMAL);
 	notify_notification_set_timeout (notify, NOTIFY_SUBSCRIPTION_TIME);
 
@@ -432,7 +468,12 @@ notify_new_message (GossipEventManager *event_manager,
 
 	contact = gossip_message_get_sender (message);
 
-	body_copy = g_strdup (gossip_message_get_body (message));
+	if (gossip_message_is_action (message)) {
+		body_copy = gossip_message_get_action_string (message);
+	} else {
+		body_copy = g_strdup (gossip_message_get_body (message));
+	}
+
 	body_stripped = g_strstrip (body_copy);
 
 	len = g_utf8_strlen (body_stripped, -1);
@@ -452,12 +493,16 @@ notify_new_message (GossipEventManager *event_manager,
 			       &show_avatars);
 
 	if (show_avatars) {
-		pixbuf = gossip_pixbuf_avatar_from_contact_scaled (contact, 32, 32);
+		pixbuf = gossip_contact_get_avatar_pixbuf (contact);
 	}
 
-	if (!pixbuf) {
+	if (pixbuf) {
+		g_object_ref (pixbuf);
+	} else {
 		/* Fall back to message icon */
-		pixbuf = gossip_pixbuf_from_stock (GOSSIP_STOCK_MESSAGE, GTK_ICON_SIZE_MENU);
+		pixbuf = gossip_stock_create_pixbuf (gossip_app_get_window (),
+						     GOSSIP_STOCK_MESSAGE, 
+						     GTK_ICON_SIZE_MENU);
 	}
 
 	title = g_strdup_printf (_("New message from %s"),
@@ -560,11 +605,11 @@ notify_disconnected_contact_foreach (GossipContact  *contact,
 }
 
 static void
-notify_protocol_disconnected_cb (GossipSession  *session,
-				 GossipAccount  *account,
-				 GossipProtocol *protocol,
-				 gint            reason,
-				 gpointer        user_data)
+notify_protocol_disconnected_cb (GossipSession *session,
+				 GossipAccount *account,
+				 GossipJabber  *jabber,
+				 gint           reason,
+				 gpointer       user_data)
 {
 	g_hash_table_remove (account_states, account);
 
@@ -578,10 +623,10 @@ notify_protocol_disconnected_cb (GossipSession  *session,
 }
 
 static void
-notify_protocol_connected_cb (GossipSession  *session,
-			      GossipAccount  *account,
-			      GossipProtocol *protocol,
-			      gpointer        user_data)
+notify_protocol_connected_cb (GossipSession *session,
+			      GossipAccount *account,
+			      GossipJabber  *jabber,
+			      gpointer       user_data)
 {
 	guint        id;
 	const gchar *account_name;
