@@ -31,7 +31,6 @@
 
 #include <libgossip/gossip-account.h>
 #include <libgossip/gossip-contact.h>
-#include <libgossip/gossip-protocol.h>
 #include <libgossip/gossip-presence.h>
 #include <libgossip/gossip-debug.h>
 #include <libgossip/gossip-conf.h>
@@ -56,11 +55,9 @@ static void sound_contact_added_cb            (GossipSession *session,
 static void sound_contact_removed_cb          (GossipSession *session,
 					       GossipContact *contact,
 					       gpointer       user_data);
-static void sound_contact_remove              (GossipContact *contact);
 
 static GHashTable    *account_states = NULL;
 static GHashTable    *contact_states = NULL;
-static gboolean       sound_disabled = FALSE;
 static GossipSession *saved_session = NULL;
 
 static gboolean
@@ -73,7 +70,7 @@ sound_protocol_timeout_cb (GossipAccount *account)
 static void
 sound_protocol_connected_cb (GossipSession  *session,
 			     GossipAccount  *account,
-			     GossipProtocol *protocol,
+			     GossipJabber   *jabber,
 			     gpointer        user_data)
 {
 	guint        id;
@@ -116,10 +113,10 @@ sound_disconnected_contact_foreach (GossipContact  *contact,
 }
 
 static void
-sound_protocol_disconnecting_cb (GossipSession  *session,
-				 GossipAccount  *account,
-				 GossipProtocol *protocol,
-				 gpointer        user_data)
+sound_protocol_disconnecting_cb (GossipSession *session,
+				 GossipAccount *account,
+				 GossipJabber  *jabber,
+				 gpointer       user_data)
 {
 	gossip_debug (DEBUG_DOMAIN, 
 		      "Protocol disconnecting for account:'%s'",
@@ -193,17 +190,12 @@ static void
 sound_contact_removed_cb (GossipSession *session,
 			  GossipContact *contact,
 			  gpointer       user_data)
-{
-	g_hash_table_remove (contact_states, contact);
-}
-
-static void
-sound_contact_remove (GossipContact *contact)
 {
 	g_signal_handlers_disconnect_by_func (contact,
 					      sound_contact_presence_updated_cb,
 					      NULL);
-	g_object_unref (contact);
+
+	g_hash_table_remove (contact_states, contact);
 }
 
 void
@@ -218,15 +210,6 @@ gossip_sound_play (GossipSound sound)
 
 	session = gossip_app_get_session ();
 
-	/* This is the internal sound enable/disable for when events
-	 * happen that we need to mute sound - e.g. changing roster
-	 * contacts.
-	 */
-	if (sound_disabled) {
-		gossip_debug (DEBUG_DOMAIN, "Play request ignored, sound currently disabled.");
-		return;
-	}
-
 	gossip_conf_get_bool (gossip_conf_get (),
 			       GOSSIP_PREFS_SOUNDS_FOR_MESSAGES,
 			       &enabled);
@@ -258,15 +241,15 @@ gossip_sound_play (GossipSound sound)
 	switch (sound) {
 	case GOSSIP_SOUND_CHAT:
 		gossip_debug (DEBUG_DOMAIN, "Triggering 'Chat' event.");
-		gnome_triggers_do (NULL, NULL, "gossip", "Chat", NULL);
+		gnome_triggers_do (NULL, NULL, PACKAGE_TARNAME, "Chat", NULL);
 		break;
 	case GOSSIP_SOUND_ONLINE:
 		gossip_debug (DEBUG_DOMAIN, "Triggering 'Online' event.");
-		gnome_triggers_do (NULL, NULL, "gossip", "Online", NULL);
+		gnome_triggers_do (NULL, NULL, PACKAGE_TARNAME, "Online", NULL);
 		break;
 	case GOSSIP_SOUND_OFFLINE:
 		gossip_debug (DEBUG_DOMAIN, "Triggering 'Offline' event.");
-		gnome_triggers_do (NULL, NULL, "gossip", "Offline", NULL);
+		gnome_triggers_do (NULL, NULL, PACKAGE_TARNAME, "Offline", NULL);
 		break;
 	default:
 		gossip_debug (DEBUG_DOMAIN, "Unknown sound type.");
@@ -275,12 +258,6 @@ gossip_sound_play (GossipSound sound)
 #endif
 }
 
-void
-gossip_sound_set_enabled (gboolean enabled)
-{
-	sound_disabled = !enabled;
-}
-
 void
 gossip_sound_init (GossipSession *session)
 {
@@ -304,7 +281,7 @@ gossip_sound_init (GossipSession *session)
 
 	contact_states = g_hash_table_new_full (gossip_contact_hash,
 						gossip_contact_equal,
-						(GDestroyNotify) sound_contact_remove,
+						(GDestroyNotify) g_object_unref,
 						(GDestroyNotify) g_object_unref);
 
 	g_signal_connect (session, "protocol-connected",
@@ -326,6 +303,10 @@ gossip_sound_finalize (void)
 {
 	g_assert (saved_session != NULL);
 
+#ifdef HAVE_GNOME
+	gnome_sound_shutdown ();
+#endif
+
 	g_signal_handlers_disconnect_by_func (saved_session,
 					      sound_protocol_connected_cb,
 					      NULL);
